#! /bin/sh
#----------------------------------------------------------------------------
# /tmp/install.sh - asterisk installation
#
# Creation:     2004-11-19  fw
# Last Update:  $Id: install.sh 9458 2019-03-17 17:42:30Z holbru $
#
# Copyright (c) 2004-2005 Fabian Wolter, fabian(at)eisfair(dot)org
# Copyright (c) 2012-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# set package name
package_name=asterisk

conffile=/etc/config.d/asterisk
defaultconffile=/etc/default.d/asterisk
#tsblcrconf=/etc/asterisk/lcr/tsblcr.cfg
#tsblcrrouting=/etc/asterisk/lcr/tsblcr.routing
plugin_menu=/var/install/menu/setup.services.asterisk.plugins.menu

master_update='2.1.10'

# ---------------------------------------------------------------------------
# create user and group
# ---------------------------------------------------------------------------
run_user='asterisk'
home_dir='/var/lib/asterisk'

installed_base=$(eisman check base 2.8.12)
    
case ${installed_base} in
old|installed)
    s_group='-r'
    ;;
esac

if ! getent group ${run_user} >/dev/null 2>&1
then
    # group_id '-r' is set by the system  
    /var/install/bin/add-group ${s_group} "${run_user}"
fi

if ! getent passwd ${run_user} >/dev/null 2>&1
then
    # system user user_id '-r', is set by the system
    /var/install/bin/add-user -r ${run_user} '*' '' ${run_user} "Asterisk PBX" ${home_dir} /bin/false 2>/dev/null
fi

# ---------------------------------------------------------------------------
# check /dev/dahdi user = asterisk
# ---------------------------------------------------------------------------
if [ -c /dev/dahdi/channel ]
then
    dev_dahdi=$( stat -c %U /dev/dahdi/channel )
    if [ "${dev_dahdi}" != "asterisk" ]
    then
        chown -R asterisk.asterisk /dev/dahdi
    fi
fi

# ---------------------------------------------------------------------------
# config file
# ---------------------------------------------------------------------------
sample_path='/usr/share/asterisk/configs'
config_path='/etc/asterisk'
config_file='adsi.conf
agents.conf
alarmreceiver.conf
amd.conf
app_mysql.conf
asterisk.adsi
asterisk.conf
ccss.conf
cdr.conf
cdr_custom.conf
cdr_manager.conf
cdr_mysql.conf
cdr_sqlite3_custom.conf
cdr_syslog.conf
cel.conf
cel_custom.conf
cel_sqlite3_custom.conf
chan_mobile.conf
chan_ooh323.conf
cli_aliases.conf
cli.conf
cli_permissions.conf
codecs.conf
dbsep.conf
dnsmgr.conf
dsp.conf
dundi.conf
enum.conf
extconfig.conf
extensions.ael
extensions.conf
extensions.lua
extensions_minivm.conf
features.conf
festival.conf
followme.conf
gtalk.conf
h323.conf
http.conf
iax.conf
iaxprov.conf
indications.conf
jabber.conf
jingle.conf
logger.conf
manager.conf
meetme.conf
mgcp.conf
minivm.conf
misdn.conf
modules.conf
musiconhold.conf
muted.conf
osp.conf
oss.conf
phone.conf
phoneprov.conf
queuerules.conf
queues.conf
res_config_mysql.conf
res_config_sqlite.conf
res_curl.conf
res_fax.conf
res_ldap.conf
res_pktccops.conf
res_stun_monitor.conf
rtp.conf
say.conf
sip.conf
sip_notify.conf
skinny.conf
sla.conf
smdi.conf
telcordia-1.adsi
udptl.conf
unistim.conf
users.conf
voicemail.conf
vpb.conf'

mkdir -p /etc/asterisk
chmod 0751 /etc/asterisk
# ---------------------------------------------------------------------------
# clear up old value from asterisk.conf
# ---------------------------------------------------------------------------
if [ -f ${config_path}/asterisk.conf ]
then
    if ! grep -q ';asterisk_conf_version 24693' ${config_path}/asterisk.conf
    then
        rm -f ${config_path}/asterisk.conf
    fi
fi

for _conf in ${config_file}
do
    if [ ! -f "${config_path}/${_conf}" ]
    then
        cp ${sample_path}/${_conf}.sample ${config_path}/${_conf}
        chmod 0600 ${config_path}/${_conf}
    fi
done




# ---------------------------------------------------------------------------
# fix musiconhold path
# ---------------------------------------------------------------------------
if grep -q '/var/lib/asterisk/mohmp3'  ${config_path}/musiconhold.conf
then
    sed -i 's#/var/lib/asterisk/mohmp3#/usr/share/asterisk/mohmp3#' ${config_path}/musiconhold.conf
fi

# ---------------------------------------------------------------------------
# Generate symbolic links: stop
# ---------------------------------------------------------------------------
#cd /etc/rc2.d
#
#ln -sf ../init.d/${package_name} K22${package_name}

#rm -f /usr/local/asterisk_webinterface/bin/mini_httpd
#ln -sf /usr/sbin/mini_httpd /usr/local/asterisk_webinterface/bin/
#rm -f /usr/local/asterisk_webinterface/bin/htpasswd
#ln -sf /usr/sbin/htpasswd /usr/local/asterisk_webinterface/bin/


# ---------------------------------------------------------------------------
# add menu
# ---------------------------------------------------------------------------
/var/install/bin/add-menu     \
        setup.services.menu   \
        setup.services.${package_name}.menu  \
        "Asterisk"

## ---------------------------------------------------------------------------
## check for installed asterisk modules and update
## ---------------------------------------------------------------------------
#_pack='alsa
#calendar
#console
#dahdi
#freetds
#odbc
#pgsql
#radius
#spandsp'
#
#for i in ${_pack}
#do
#    if [ -f /var/install/packages/asterisk-${i} ]
#    then
#        (
#        /var/install/bin/install-package -y -p asterisk-${i} -v 2.5.0
#        )
#    fi
#done
#
#__pack='lcr
#sounds-de
#sounds-fr
#chan-capi
#chan-sccp
#moh-base'
#
#for i in ${__pack}
#do
#    if [ -f /var/install/packages/asterisk-${i} ]
#    then
#        (
#        /var/install/bin/install-package -y -p asterisk-${i} -v 2.6.0
#        )
#    fi
#done

# ---------------------------------------------------------------------------
# Create default config
# ---------------------------------------------------------------------------
if [ -f "/etc/config.d/${package_name}" ]
then
    update=true
fi

/var/install/config.d/${package_name}-update.sh

# remove configuration update script
rm -f /var/install/config.d/${package_name}-update.sh

mkdir -p /var/log/asterisk/cdr-csv
mkdir -p /var/spool/asterisk/outgoing
mkdir -p /usr/share/asterisk/keys

chown -R asterisk.asterisk /var/log/asterisk
chown -R asterisk.asterisk /var/spool/asterisk
chown -R asterisk.asterisk /var/lib/asterisk
chmod 0750 /var/log/asterisk
chmod 0750 /var/spool/asterisk
chmod 0750 /var/lib/asterisk
# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
#if [ -f /tmp/tsblcr.conf ]
#then
#    mv /tmp/tsblcr.conf $tsblcrconf
#fi
#
#if [ -f /tmp/tsblcr.routing ]
#then
#    mv /tmp/tsblcr.routing $tsblcrrouting
#fi

if [ ! -f $plugin_menu ]
then
{
echo "<title>Asterisk Pluginmenu</title>
<package>asterisk</package>"
}>/var/install/menu/setup.services.asterisk.plugins.menu
fi

# Remove not completed web based configuration
rm -fr usr/local/asterisk_webinterface/htdocs/modules/config/

# ---------------------------------------------------------------------------
# restore http settings
# ---------------------------------------------------------------------------

local_path='/tmp/asterisk'
web_path='/var/lib/asterisk/webinterface'
if [ -f ${local_path}/httpd.pem ]
then
    cp -a ${local_path}/httpd.pem ${web_path}
fi
if [ -f ${local_path}/httpd.rnd ]
then
    cp -a ${local_path}/httpd.rnd ${web_path}
fi
if [ -f ${local_path}/.htpasswd ]
then
    cp -a ${local_path}/.htpasswd ${web_path}/htdocs
fi

rm -rf ${local_path}
rm -rf /usr/local/asterisk_webinterface

## ---------------------------------------------------------------------------
## check installed asterisk sub-packages
## ---------------------------------------------------------------------------
#sub_packages='calendar
#console
#dahdi
#freetds
#odbc
#pgsql
#radius
#spandsp'
#
#for _pack in ${sub_packages}
#do
#    installed_version=$( /var/install/bin/check-version "asterisk-${_pack}" "${master_update}" )
#
#    case ${installed_version} in
#    new)
#        /var/install/bin/install-package -y -p "asterisk-${_pack}" -v "${master_update}"
#        ;;
#    *)
#        :  # nothing to do
#        ;;
#    esac
#
#done

# ---------------------------------------------------------------------------
# start package with update
# ---------------------------------------------------------------------------
if ${update:-false}
then
    . /etc/config.d/${package_name}
#    /etc/init.d/asterisk status >/dev/null

    if [ "${ASTERISK_WEBINTERFACE_START}" = "yes" ]
    then
        if [ -z "${ASTERISK_WEBINTERFACE_CERTIFICATE}" ]
        then
            old_pem='/var/lib/asterisk/webinterface/httpd.pem'
            new_pem='/usr/local/ssl/certs/httpd.pem'
            if [ -f "${old_pem}" ]
            then
                if [ ! -f "${new_pem}" -o ! -L "${new_pem}" ]
                then
                    cp -a ${old_pem} ${new_pem}

                    sed -i "s|ASTERISK_WEBINTERFACE_CERTIFICATE=.*$|ASTERISK_WEBINTERFACE_CERTIFICATE='httpd.pem'|" /etc/config.d/${package_name}
                else
                    sed -i "s|ASTERISK_WEBINTERFACE_START=.*$|ASTERISK_WEBINTERFACE_START='no'|" /etc/config.d/${package_name}

                    . /var/install/include/eislib

                    echo
                    mecho --warn "The certificate handling has changed"
                    mecho --warn "Please check the new parameter"
                    mecho        "ASTERISK_WEBINTERFACE_CERTIFICATE"
                    mecho --warn "to set the correct certificate name."
                    mecho --info "ASTERISK_WEBINTERFACE_START -> 'no'"
                    echo
                    anykey
                fi
            fi
        fi
    fi

    /var/install/config.d/${package_name}.sh --quiet

    if [ "$START_ASTERISK" = "yes" ]
    then
        /etc/init.d/asterisk start
    fi

#    /etc/init.d/${package_name} start
fi

# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
