#! /bin/sh
#----------------------------------------------------------------------------
# /tmp/preinstall.sh - Asterisk preinstallation
#
# Creation:     2004-11-19  fw
# Last Update:  $Id: preinstall.sh 8388 2015-02-15 22:36:30Z holbru $
#
# Copyright (c) 2005 Fabian Wolter, fabian(at)eisfair(dot)org
# Copyright (c) 2012-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

. /var/install/include/eislib


package_name=asterisk

initscript=/etc/init.d/${package_name} 

# ---------------------------------------------------------------------------
# 
# ---------------------------------------------------------------------------

if [ -f $initscript ]
then
    if $initscript status >/dev/null
    then
        mecho --warn "ATTENTION: During the installation proccess the Asterisk call system won't be available."
        mecho --warn "           Asterisk will shutdown after pressing ENTER!"
        /var/install/bin/anykey

        sh $initscript stop
    fi
fi

# ---------------------------------------------------------------------------
# update system
# ---------------------------------------------------------------------------
if [ -f "/var/install/deinstall/${package_name}" ]
then
    inst_version=$( /var/install/bin/check-version ${package_name} 2.1.10 )
    case ${inst_version} in
    new)
        local_path='/usr/local/asterisk_webinterface'
        mkdir -p /tmp/asterisk 
        if [ -f ${local_path}/httpd.pem ] 
        then
            cp -a ${local_path}/httpd.pem /tmp/asterisk
        fi 
        if [ -f ${local_path}/httpd.rnd ]
        then
            cp -a ${local_path}/httpd.rnd /tmp/asterisk
        fi 
        if [ -f ${local_path}/htdocs/.htpasswd ]
        then
            cp -a ${local_path}/htdocs/.htpasswd /tmp/asterisk
        fi
        ;;
    *)
        : # nothing to do
        ;;
    esac

    inst_version=$(/var/install/bin/check-version ${package_name} 2.1.7)
    case ${inst_version} in
    new)
        cp /etc/config.d/${package_name} /etc/config.d/${package_name}-old

        /var/install/bin/del-package -p ${package_name}
        rm -rf /usr/lib/${package_name}/modules/

        mv /etc/config.d/${package_name}-old /etc/config.d/${package_name}
        ;;
    *)
        /var/install/bin/del-package -p ${package_name} --update
        ;;
    esac
fi

# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
