#!/usr/bin/php -q
<?php
/**
 *---------------------------------------------------------------------------
 * /usr/share/asterisk/agi-bin/getName.php
 *
 * Creation:     2005-10-28  fw
 * Last Update:  $Id: getName.php 8393 2015-02-16 21:50:50Z holbru $
 *
 * Copyright (c) 2005 Fabian Wolter, fabian(at)eisfair(dot)org
 * Copyright (c) 2014-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------
 */

$t1 = microtime(TRUE);

$local_file = "/public/phonelist.txt";

ob_implicit_flush(false);
set_time_limit(2);
error_reporting(0);

include '/var/lib/asterisk/config/include/reverseLookup.php';

$stdin = fopen('php://stdin', 'r');

foreach ($_SERVER["argv"] as $argv) {
    $it = explode("=", $argv);
    $ARGS[$it[0]] = $it[1];
}

while (!feof($stdin)) {
    $temp = fgets($stdin);
    $temp = str_replace("\n", "", $temp);

    $s = explode(":", $temp);
    $agivar[$s[0]] = trim($s[1]);

    if (($temp == "") || ($temp == "\n")) {
        break;
    }
}

echo "VERBOSE \"CallerID: " . addslashes($agivar["agi_callerid"]) . "\"\n";

if (preg_match("#\"(.*)\" <(.*)>#", $agivar["agi_callerid"], $matches)) {
    $cidName = $matches[1];
    $cidNum = $matches[2];

    if ($cidNum != $cidName) {
        echo "VERBOSE \"Name already set to " . $cidName . ". Skipping inverse search...\"\n";

        $cidNum = addZero($cidNum);

        setCID($cidName, $cidNum);

        if (is_integer(strpos($cidName, "@"))) {
            list ($number) = explode("@", $cidName);

            if (!is_integer($number))
                $number = 0; # Wenn der Userpart nicht numerisch ist (z.B. bei Web.de), wird die CallerIDNum auf 0 gesetzt. (Manche ISDN-Telefone stellen den Text nur dar, wenn eine Nummer gesetzt ist)

            setCID($cidName, $number);
        }
        quit();
    }

    $number = addZero($cidNum);
}
elseif (preg_match("#<(.*)>#", $agivar["agi_callerid"], $matches)) {
    $cidNum = $matches[1];
    $number = addZero($cidNum);
}
elseif (is_numeric($agivar["agi_callerid"])) {
    $number = addZero($agivar["agi_callerid"]);
} else {
    $number = $agivar["agi_callerid"];
}

if ($number == "" || stripos($number, "unknown") !== FALSE) {
    echo "VERBOSE \"Skipping, no CallerID transmitted...\"\n";
    $number = "";
    quit();
}
elseif ($agivar["agi_callerid"] != $number) {
    echo "VERBOSE \"Telephone number: $number\"\n";
}

//if ($number {
//    0 }
//!= 0) {
//    echo "VERBOSE \"Invalid telephone number\"\n";
//    quit();
//}

if (isset ($ARGS["path"])) {
    if (is_file($local_file)) {
        $file = file($local_file);
        for ($i = 0; $i <= count($file); $i++) {

            $data = explode("\t", $file[$i]);

            if ($data[0] == $number) {

                $name = trim($data[1]); # remove LF

                echo "VERBOSE \"CallerIDName by local telephonenumber list: $name\"\n";

                setCID($name, $number);

                quit();
            }
        }
        echo "VERBOSE \"No data found about $number in local telephonenumber list\"\n";
    }
}
elseif ($ARGS["host"]) {
    if (!mysql_connect($ARGS["host"], $ARGS["user"], $ARGS["password"])) {
        echo "VERBOSE \"ERROR: MySQL connect: " . mysql_error() . "\"\n";
    } else {
        if (!mysql_select_db($ARGS["dbname"])) {
            echo "VERBOSE \"ERROR: MySQL select:" . mysql_error() . "\"\n";
        } else {
            list ($name, $mysqlError) = lookupName($number);
            if ($name) {
                setCID($name, $number);
                echo "VERBOSE \"CallerIDName by MySQL database: $name\"\n";
                quit();
            }
            elseif ($mysqlError) {
                echo "VERBOSE \"ERROR: MySQL: $mysqlError\"\n";
            } else
                echo "VERBOSE \"No data found about $number in MySQL database\"\n";
        }
    }
}

if ($ARGS["local"] != "yes") {

    if (!($fp = fsockopen("www.dasoertliche.de", 80))) {
        echo "VERBOSE \"Connection to dasortliche.de failed!\"\n";
        quit();
    }

    $body = "form_name=search_inv&la=de&page=RUECKSUCHE&context=RUECKSUCHE&action=SUCHEN&buc=&ph=$number&image.x=83&image.y=8";

    $head = "POST /Controller HTTP/1.1\r\n".
    "Host: www1.dasoertliche.de\r\n".
    "User-Agent: eisfair Asterisk v1.1.x\r\n".
    "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5\r\n".
    "Accept-Language: de-de,de;q=0.8,en-us;q=0.5,en;q=0.3\r\n".
    "Accept-Encoding: gzip,deflate\r\n".
    "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7\r\n".
    "Keep-Alive: 300\r\n".
    "Connection: close\r\n".
    "Content-Type: application/x-www-form-urlencoded\r\n".
    "Content-Length: " . strlen($body);

    fputs($fp, $head . "\r\n\r\n" . $body . "\r\n\r\n");

    while (!feof($fp)) {
        $line = fgets($fp);

        if (preg_match('~class="entry">([^<]*)</a>~', $line, $matches)) {
            $name = trim($matches[1]);

            echo "VERBOSE \"CallerIDName by dasoertliche.de: $name\"\n";
            setCID($name, $number);

            if (isset ($ARGS["path"])) {
                if ($fp_local_file = fopen($local_file, "a")) {
                    fwrite($fp_local_file, "$number\t$name\n");
                    fclose($fp_local_file);
                } else {
                    echo "VERBOSE \"Cannot open local telephonenumber list for writing results\"\n";
                }
            }
            elseif ($ARGS["host"]) {
                $result = mysql_query("INSERT INTO " . $ARGS["table"] . " (number, name) VALUES('$number', '$name')");
                if (!$result) {
                    echo "VERBOSE \"ERROR: Couldn't write to database!\"\n";
                }
                mysql_close();

            }
            quit();
        }
    }

    echo "VERBOSE \"No data found about $number at dasortliche.de\"\n";
}

setCID("", $number);

quit();
?>
