#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/asterisk.sh - configuration generator script for Asterisk
#
# Creation:     2004-11-07 fw
# Last Update:  $Id: asterisk.sh 9346 2016-09-07 19:39:39Z holbru $
#
# Copyright (c) 2005 Fabian Wolter, fabian(at)eisfair(dot)org
# Copyright (c) 2012-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------


package_name='asterisk'

conffile=/etc/config.d/${package_name}

ast_logger=/etc/asterisk/logger.conf
ast_voicemail=/etc/asterisk/voicemail.conf
ast_system=/etc/asterisk/system.conf
ast_dahdi=/etc/asterisk/chan_dahdi.conf
ast_modules=/etc/asterisk/modules.conf
conf_bin=/var/install/config.d
cronFile=/var/cron/etc/root/asterisk
cronExec=/etc/asterisk/lcr/tsblcrupdate.php
cronConf=/etc/asterisk/lcr/tsblcr.cfg
cronRouting=/etc/asterisk/lcr/tsblcr.routing
webinterface=/var/lib/asterisk/webinterface
cert=${webinterface}/httpd.pem
certconf=${webinterface}/etc/httpd.cnf
httpd_conf=${webinterface}/etc/httpd.conf
bchans=2

# require versions
chan_dahdi_ver=2.1.4
chan_capi_ver=2.1.2
#mini_http_ver=2.2.5
lcr_ver=2.1.0
certs_ver=1.4.1

tty=$(tty)

# In Sekunden. Wenn innerhalb dieser Zeit niemand ans Telefon
# geht bekommt der Anrufer ein "Besetzt". Tritt allerdings ausser
# Kraft, wenn ..._VBOX_ANSWER_AFTER gesetzt ist und ..._VBOX='yes'
timeout_std=60

. $conffile
. /var/install/include/eislib

# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
debug()
{
    if [ "$ASTERISK_DO_DEBUG" = "yes" ]
    then
        echo ">>>>>> $*" >${tty}
    fi
 #   echo -n "."
}

# ---------------------------------------------------------------------------
# asterisk start
# ---------------------------------------------------------------------------
asterisk_start()
{
    cd /etc/rc2.d
    rm -f S??${package_name}
    ln -sf ../init.d/${package_name} S78${package_name}
}

# ---------------------------------------------------------------------------
# asterisk stop
# ---------------------------------------------------------------------------
asterisk_stop()
{
    rm -f /etc/rc2.d/S??${package_name}
}

# ---------------------------------------------------------------------------
# create webinterface access
# ---------------------------------------------------------------------------
create_webinterface_access()
{
    if [ "${ASTERISK_WEBINTERFACE_FORCE_SSL}" = "yes" ]
    then
        check_package "certs" "${certs_ver}"
        ret="${?}"
        if [ ${ret} -eq 0 ]
        then
            create_cert
            create_httpd_conf ssl
        fi
    else
        create_httpd_conf no
    fi
}

# ---------------------------------------------------------------------------
# create httpd.conf
# ---------------------------------------------------------------------------
create_httpd_conf()
{
    _ssl="${1}"

    cat >${httpd_conf} <<EOF
port=3002
cgipat=**.cgi|**.php
pidfile=${webinterface}/httpd.pid
user=root
EOF

    if [ "${_ssl}" = "ssl" ]
    then
        {
        echo 'SSL'
        echo 'certfile=/usr/local/ssl/certs/asterisk.pem'
        } >>${httpd_conf}
    fi

    chown asterisk.asterisk ${httpd_conf}
}
# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
write_htpasswd()
{

    # password syntax
    # openssl passwd -crypt myPassword
    # http://httpd.apache.org/docs/2.4/misc/password_encryptions.html

    passwdlib=/var/lib/asterisk/htpasswd

    >${webinterface}/htdocs/.htpasswd

    idx=1
    while [ $idx -le $ASTERISK_WEBINTERFACE_N ]
    do
        eval user='$ASTERISK_WEBINTERFACE_'$idx'_USER'
        eval passwd='$ASTERISK_WEBINTERFACE_'$idx'_PASSWD'

        if [ -f ${webinterface}/htdocs/.htpasswd ]
        then
            htpasswdParam=""
        else
            htpasswdParam="-c"
        fi

        echo ${passwd} | ${passwdlib} ${htpasswdParam} ${webinterface}/htdocs/.htpasswd "${user}" >/dev/null

        idx=$((${idx} + 1))
    done
}
# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
create_cert ()
{

#    if [ ! -f $cert ]
#    then
#        mecho --info "Creating new certificate for Asterisk Webinterface ..."
#        echo
#
#        # run interactive or not?
#        if [ "`get_printmode`" = "tty" ]
#        then
#            mecho --warn "Please answer the following questions carefully"
#            mecho --warn "You are asked only once"
#            echo
#            /usr/bin/openssl req -new -x509 -days 365 -nodes \
#                -config $certconf -out $cert -keyout $cert
#            echo
#        else
#            /usr/bin/openssl req -new -x509 -days 365 -nodes -batch \
#                -config $certconf -out $cert -keyout $cert
#        fi
#
#        # check certificate
#        echo "Checking certificate settings ..."
#        /usr/bin/openssl x509 -subject -dates -fingerprint -noout -in $cert
#
#        chmod 600 $cert
#    fi

    cert_path='/usr/local/ssl/certs'
    cert_name="${ASTERISK_WEBINTERFACE_CERTIFICATE}"
    cert_link='asterisk.pem'
    cert_rehash='/usr/bin/ssl/c_rehash'

    if [ -f ${cert_path}/${cert_name} ]
    then
        cd ${cert_path}
        rm -f ${cert_link}
        ln -sf ${cert_name} ${cert_link}
        # rehash certificate
        ${cert_rehash} ${cert_path} >/dev/null 2>&1
#    else
#        sed -i "s|ASTERISK_WEBINTERFACE_START=.*$|#ASTERISK_WEBINTERFACE_START='no'|" /etc/config.d/${package_name}
    fi
}
# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
add_cron () {
debug add_cron
{
    echo "# Do not edit this file, edit /etc/config.d/asterisk"
    echo "# Creation date: ${EISDATE}  ${EISTIME}"
    echo "$ASTERISK_LCR_CRON $cronExec $cronConf $cronRouting >/dev/null 2>&1"
}>$cronFile
    # create cron
    /var/install/config.d/cron >/dev/null 2>&1
}
# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
del_cron () {
debug del_cron
    rm -f $cronFile
    # del cron
    /var/install/config.d/cron >/dev/null 2>&1
}
# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
write_modules_conf () {
debug write_modules_conf
{
    echo "[global]"
    echo "chan_capi.so=yes"

    echo "[modules]"
    echo "autoload=yes"
    echo "noload => pbx_gtkconsole.so"
    echo "noload => pbx_kdeconsole.so"
    echo "noload => app_intercom.so"
    echo "noload => chan_alsa.so"
    echo "noload => chan_oss.so"
    echo "noload => res_odbc.so"
    echo "noload => cdr_odbc.so"
    echo "noload => cdr_custom.so"
    echo "noload => pbx_ael.so"
    echo 'noload => res_config_ldap.so'

    ### Asterisk addons
    echo "noload => app_addon_sql_mysql.so"
    echo "noload => app_saycountpl.so"
    echo "noload => cdr_addon_mysql.so"
    echo "noload => format_mp3.so"
    echo "noload => res_config_mysql.so"

    if [ "${ASTERISK_LOAD_CAPI}" != "yes" ]
    then
        echo "noload => chan_capi.so"
#         echo "noload => app_capiCD.so"
#         echo "noload => app_capiECT.so"
#         echo "noload => app_capiHOLD.so"
#         echo "noload => app_capiMCID.so"
#         echo "noload => app_capiNoES.so"
#         echo "noload => app_capiRETRIEVE.so"
    fi

    if [ "${ASTERISK_LOAD_DAHDI}" != "yes" ]
    then
        echo "noload => chan_dahdi.so"
    fi

    idx=1
    while [ ${idx} -le ${ASTERISK_PHONES_N} ]
    do
        eval active='${ASTERISK_PHONES_'${idx}'_ACTIVE}'

        if [ "${active}" = "yes" ]
        then
            eval type='${ASTERISK_PHONES_'${idx}'_TYPE}'
            if [ "${type}" = "sccp" ]
            then
                idx=$((${{idx}} + 1))
                sccp_phone=true
                continue
            fi
        fi
        idx=$((${idx} + 1))
    done

    if "${sccp_phone:-false}"
    then
        if [ -e "/usr/lib/asterisk/modules/chan_sccp.so"  ]
        then
            echo "noload => chan_skinny.so"
        fi
    fi

}>$ast_modules
}
# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
write_voicemail_conf () {
debug write_voicemail_conf
{
    echo "[general]"
    echo "format=wav"
    echo "serveremail=$ASTERISK_VMAIL_FROM_ADDR"
    echo "fromstring=$ASTERISK_VMAIL_FROM_NAME"
    echo "maxsecs=$ASTERISK_VBOX_MAX_MSG_LENGTH"
    echo "minsecs=3"
    echo "attach=yes"
    echo "skipms=3000"
    echo "maxsilence=2"
    echo "silencethreshold=128"
    echo "maxlogins=3"
    echo


    echo "[zonemessages]"
    echo "de=Europe/Berlin|'vm-received' q 'digits/at' H M"
    echo
    echo "[default]"
    echo "; Internal voice boxes"

    idx=1
    while [ $idx -le $ASTERISK_PHONES_N ]
    do
        eval active='${ASTERISK_PHONES_'${idx}'_ACTIVE}'

        if [ "${active}" = "yes" ]
        then
            eval msn='$ASTERISK_PHONES_'$idx'_MSN'
            eval pin='$ASTERISK_PHONES_'$idx'_PIN'
            eval mail='$ASTERISK_PHONES_'$idx'_VBOX_MAIL'
            eval name='$ASTERISK_PHONES_'$idx'_OUTGOING_NAME'
            eval vbox='$ASTERISK_PHONES_'$idx'_VBOX'

            if [ "$vbox" != "yes" ]
            then
                idx=$((${idx} + 1))
                continue
            fi

            echo "$msn => $pin,$name,$mail,,tz=de"

            mkdir -p /var/spool/asterisk/voicemail/default/$msn/Old
            mkdir -p /var/spool/asterisk/voicemail/default/$msn/INBOX
            mkdir -p /usr/share/asterisk/sounds/voicemail/default/$msn
        fi
        idx=$((${idx} + 1))
    done

    echo
    echo "; Additional voice boxes"
    echo

    idx=1
    while [ $idx -le $ASTERISK_VBOX_N ]
    do
        eval active='${ASTERISK_VBOX_'${idx}'_ACTIVE}'

        if [ "${active}" = "yes" ]
        then
            eval number='$ASTERISK_VBOX_'$idx'_NUMBER'
            eval pin='$ASTERISK_VBOX_'$idx'_PIN'
            eval mail='$ASTERISK_VBOX_'$idx'_MAIL'
            eval name='$ASTERISK_VBOX_'$idx'_OWNER_NAME'

            echo "$number => $pin,$name,$mail,,tz=de"

            mkdir -p /var/spool/asterisk/voicemail/default/$number
            mkdir -p /var/lib/asterisk/sounds/voicemail/default/$number
        fi
        idx=$((${idx} + 1))
    done
}>$ast_voicemail
}

# ---------------------------------------------------------------------------
# check chan_capi
# ---------------------------------------------------------------------------
check_chan_capi ()
{
    check_package 'asterisk-chan-capi' "${chan_capi_ver}"
    ret="${?}"
    return ${ret:-0}
}

# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
check_capi_modules ()
{
debug write_capi_conf

    capi_file=/etc/capi.d/${package_name}

    if [ "${ASTERISK_LOAD_CAPI}" = "yes" ]
    then
        if ! ${quietmode:-false}
        then
            check_chan_capi

            if [ ! -f /usr/lib/asterisk/modules/chan_capi.so ]
            then
                echo y | eisman uninstall --force asterisk-chan-capi
                check_chan_capi
            fi
        fi

        mkdir -p /etc/capi.d
        echo "Don't remove me" >${capi_file}
        chmod 0620  ${capi_file}
    else
        rm -f ${capi_file}
    fi
}

# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
check_fx_type ()
{
    if [ "$type" = "none" ]
    then
        idx=$((${idx} + 1))
        channel=$((${channel} + ${ports}))
        continue
    elif [ "$type" = "fxs" ]
    then
        type=fxo
    elif [ "$type" = "fxo" ]
    then
        type=fxs
    fi
}

# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
write_logger_conf ()
{
debug write_logger_conf
    {
    echo "[general]"
    echo "[logfiles]"
    echo "console => notice,warning,error"
    echo "messages => notice,warning,error"

    if [ "$ASTERISK_LOG_INTO_FILE" = "yes" ]
    then
        echo "full => notice,warning,error,debug,verbose"
    fi
    echo
    }>$ast_logger
}

# ---------------------------------------------------------------------------
# check chan_dahdi
# ---------------------------------------------------------------------------
check_chan_dahdi ()
{
    check_package 'asterisk-dahdi' "${chan_dahdi_ver}"
    ret="${?}"
    return ${ret:-0}
}

# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
check_dahdi_modules ()
{
    dahdi_file=/etc/dahdi.d/${package_name}

    if [ "${ASTERISK_LOAD_DAHDI}" = "yes" ]
    then
        if ! ${quietmode:-false}
        then
            check_chan_dahdi

            if [ ! -f /usr/lib/asterisk/modules/chan_dahdi.so ]
            then
                echo y | eisman uninstall --force asterisk-dahdi
                check_chan_dahdi
            fi
        fi

        mkdir -p /etc/dahdi.d
        echo "Don't remove me" >${dahdi_file}
        chmod 0620  ${dahdi_file}
    else
        rm -f ${dahdi_file}
    fi
}

# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
write_zapata_conf ()
{
debug write_zapata_conf

    check_dahdi_modules

    {
    echo "#include dahdi-channels.conf"
    echo "[channels]"
    echo "language=$ASTERISK_LANGUAGE"
    echo "switchtype=euroisdn"
    echo "pridialplan=local"
    echo "prilocaldialplan=local"
    echo "echocancel=yes"
    echo "immediate=no"
    echo "overlapdial=yes"
    echo "nationalprefix=0"
    echo "internationalprefix=00"
    echo

    channel=1
    group=1

    idx=1
    while [ $idx -le $ASTERISK_HFCS_GROUP_N ]
    do
        eval ptp='$ASTERISK_HFCS_GROUP_'$idx'_PTP'
        eval bus='$ASTERISK_HFCS_GROUP_'$idx'_PROVIDE_S0_BUS'
        eval ports='$ASTERISK_HFCS_GROUP_'$idx'_PORTS'
        #eval bchans='$ASTERISK_HFCS_GROUP_'$idx'_BCHANNELS'#

        if [ "$ptp" = "yes" -a "$bus" = "yes" ]
        then
            signalling="bri_net"
        elif [ "$ptp" = "yes" -a "$bus" = "no" ]
        then
            signalling="bri_cpe"
        elif [ "$ptp" = "no" -a "$bus" = "yes" ]
        then
            signalling="bri_net_ptmp"
        elif [ "$ptp" = "no" -a "$bus" = "no" ]
        then
            signalling="bri_cpe_ptmp"
        fi

        echo "signalling=$signalling"
        echo "context=default"

        port=1
        while [ $port -le $ports ]
        do
            echo "group=$group"

            echo "channel => $channel-$((${channel} + 1))"

            port=$((${port} + 1))
            channel=$((${channel} + ${bchans} + 1))
        done

        echo

        group=$((${group} + 1))
        idx=$((${idx} + 1))
    done

    idx=1
    while [ $idx -le $ASTERISK_FX_GROUP_N ]
    do
        eval type='$ASTERISK_FX_GROUP_'$idx'_TYPE'
        eval ports='$ASTERISK_FX_GROUP_'$idx'_PORTS'

        check_fx_type

        echo "signalling=${type}_ks"
        echo "context=outgoing"

        port=1
        while [ $port -le $ports ]
        do
            echo "group=$group"
            echo "channel => $channel"

            port=$((${port} + 1))
            channel=$((${channel} + 1))
        done

        echo

        group=$((${group} + 1))
        idx=$((${idx} + 1))
    done

    }>${ast_dahdi}
}
# ---------------------------------------------------------------------------
# Die Funktion wird in zwei Teile gesplittet,
# da $channel immer inkrementiert werden muss,
# aber der FXX-Teil als erstes in die Konfigurationsdatei
# geschrieben werden muss.
# ---------------------------------------------------------------------------
write_zaptel_conf ()
{
debug write_zaptel_conf
    {
    echo "loadzone=nl"
    echo "defaultzone=nl"
    echo

    channel=1

    idx=1
    span=1
    while [ $idx -le $ASTERISK_HFCS_GROUP_N ]
    do
        eval ptp='$ASTERISK_HFCS_GROUP_'$idx'_PTP'
        eval ports='$ASTERISK_HFCS_GROUP_'$idx'_PORTS'
        #eval bchans='$ASTERISK_HFCS_GROUP_'$idx'_BCHANNELS'

        port=1
        while [ $port -le $ports ]
        do
            echo "span=$port,1,3,ccs,ami"
            echo "bchan=$channel-$((${channel} + 1))"
            echo "dchan=$((${channel} + 2))"

            port=$((${port} + 1))
            channel=$((${channel} + ${bchans} + 1))
            span=$((${span} + 1))
        done

        echo

        idx=$((${idx} + 1))
    done

    idx=1
    while [ $idx -le $ASTERISK_FX_GROUP_N ]
    do
        eval type='$ASTERISK_FX_GROUP_'$idx'_TYPE'
        eval ports='$ASTERISK_FX_GROUP_'$idx'_PORTS'

        check_fx_type

        port=1
        while [ $port -le $ports ]
        do
            echo "${type}ks=$channel"

            port=$((${port} + 1))
            channel=$((${channel} + 1))
        done

        echo

        idx=$((${idx} + 1))
    done
    }>${ast_system}
}

# ---------------------------------------------------------------------------
# set config filemode
# ---------------------------------------------------------------------------
set_config_filemode()
{
    for _conf in $(find /etc/asterisk -maxdepth 1 -type f -name '*')
    do
        chown asterisk.asterisk ${_conf}
        chmod 0600 ${_conf}
    done
}

# ---------------------------------------------------------------------------
# set parameter to no
# ---------------------------------------------------------------------------
set_parameter_no()
{
    # ${1} rewrite Parameter from 'yes' to 'no'
    # set parameter from 'yes' to 'no'
    sed -i "s/^${1}=.*/${1}='no'/" ${config_file}
}

# ---------------------------------------------------------------------------
# check package
# ---------------------------------------------------------------------------
check_package()
{
    local package="${1}"
    local version="${2}"
    local retvar

    local installed=$(eisman check "${package}" ${version})

    if [ "${installed}" = "not-installed" ] ||
       [ "${installed}" = "new" ]
    then
        if ! ${quietmode:-false}
        then
            if [ -n "${version}" ]
            then
                version="=${version}"
            fi

            eisman install --auto --unstable ${package}${version}
            retvar=${?}
        fi
    else
        retvar=0
    fi

    return  ${retvar:-0}
}

# ---------------------------------------------------------------------------
#
# ---------------------------------------------------------------------------
main()
{
    if [ -f /public/phonelist.txt ]
    then
        chmod 0666 /public/phonelist.txt
    fi

    quietmode=false

    while [ ${#} -gt 0 ]
    do
        case ${1} in
        --quiet)
            quietmode=true
            shift
            ;;
        esac
    done

    case "${START_ASTERISK}" in
    yes)
        asterisk_start
        check_capi_modules
        check_dahdi_modules

        if [ "${ASTERISK_CONFIG_USE}" = "yes" ]
        then
            #create_symlinks
            write_modules_conf
            write_zapata_conf
            #write_zaptel_conf
            write_voicemail_conf
            write_logger_conf


            if [ "${ASTERISK_WEBINTERFACE_START}" = "yes" ]
            then
                create_webinterface_access
                write_htpasswd
            fi

            /var/install/config.d/asterisk.php

            set_config_filemode


            if [ "$ASTERISK_LCR" = "yes" ]
            then
                check_package 'asterisk-lcr'  "${lcr_ver}"
                ret="${?}"

                if [ ${ret} -eq 0 ]
                then
                    add_cron
                fi
            else
                del_cron
            fi
        fi
        ;;
    no)
        asterisk_stop
        del_cron
        ;;
    esac

    exit 0
}

# ---------------------------------------------------------------------------
# call function main
# ---------------------------------------------------------------------------
main "${@}"

# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
