<?php
/**
 *---------------------------------------------------------------------------
 * /var/lib/asterisk/config/configParse.php - Contains function to parse the shell configuration files
 *
 * Creation:     2005-10-28  fw
 * Last Update:  $Id: configParsing.php 8361 2015-02-15 20:05:35Z holbru $
 *
 * Copyright (c) 2005 Fabian Wolter, fabian(at)eisfair(dot)org
 * Copyright (c) 2014-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------
 */

function read_vars($file) {
    global $debug;

    $fp = fopen($file, 'r') or die('Error when opening '.$file."\n");

    unset ($max_level_index);
    $level = 0;
    $lastlevel = 0;
    $lastvalue = 0;
    $lastvarname = '';

    while (!feof($fp)) {
        unset ($matches);
        $line = fgets($fp);

        if (preg_match("/^([^[:space:]#]*)='([^']*)'[[:space:]]*(.*)$/is", $line, $matches));
        /**
         * Wenn der Variablen-Wert nicht mit " eingeschlossen ist,
         * wird versucht ' zu benutzen
         */
        elseif (preg_match('/^([^[:space:]#]*)="([^"]*)"[[:space:]]*(.*)$/is', $line, $matches));
        else {
           /**
            * Falls die Zeile keine Variablen enthaelt wird sie trotzdem in
            * $all_vars geschrieben, fuer das neu schreiben der config
            */
           $all_vars[]['comment'] = $line;
            continue;
        }

        $varname  = $matches[1];
        $varvalue = $matches[2];
        $comment  = trim($matches[3]);

        if (empty ($varname))
            continue;

        unset ($matches);

        $matches = preg_split('/_([0-9]+)[_]?/', $varname, -1, PREG_SPLIT_DELIM_CAPTURE);

  //      print_r($matches);
  //      echo count($matches)."\n";

        if (count($matches) == 0) {
            continue;
        }

        /**
         * NAME _ <NUM> _ NAME _ <NUM> _ NAME ...
         * level 0 = matches 1
         * level 1 = matches 3
         * level 2 = matches 5
         * level 3 = matches 7
         * --> level = (matches - 1) / 2
         */

        $level = (count($matches) - 1) / 2;

        if ($level > $lastlevel) {

            /**
             * vor dem Wechsel in eine hoehere Hierarchieebene muss
             * eine _N Variable gestanden sein. Wir merken uns den Wert.
             * ggf. Namen pruefen!
             */
            if (preg_match('/_N$/', $lastvarname)) {
                $max_level_index[$lastlevel] = $lastvalue;
            }
            else {
                /**
                 * Fehlerhafte Konfigurationsdatei! Fehler beim Parsen!
                 */
                $max_level_index[$lastlevel] = 0;
            }
        }

        $lastlevel   = $level;
        $lastvalue   = $varvalue;
        $lastvarname = $varname;

        /* fuer was auch immer das notwendig ist... */
        $all_vars[$varname] = $varvalue;

        /* Variable zuweisen */
        for ($idx = 0; $idx < $level; $idx++) {
            if ($matches[$idx * 2 + 1] > $max_level_index[$idx])
                /**
                 * wichtig! zwei Ebenen verlassen!
                 */
                continue 2;
        }

        switch($level) {
        case 0:
             $VARS[$varname] = $varvalue;
             break;
        case 1:
             $VARS[$matches[0]][$matches[1]][$matches[2]] = $varvalue;
             break;
        case 2:
             $VARS[$matches[0]][$matches[1]][$matches[2]]
                  [$matches[3]][$matches[4]] = $varvalue;
             break;
        case 3:
             $VARS[$matches[0]][$matches[1]][$matches[2]]
                  [$matches[3]][$matches[4]][$matches[5]]
                  [$matches[6]] = $varvalue;
             break;
        }

    }

 //   print_r($VARS);

    $VARS = removeInactive($VARS);

    if ($debug)
        debug(print_r($all_vars, true));
    #print_r($VARS);
    #print_r($max_value);
    #if($debug) debug(print_r($comments, TRUE));

    if (empty($comments)) {
        $comments = '';
    }

    /**
     * $VARS enthaelt nur die Variablen, die tatsaechlich ausgewertet
     * werden sollen (ASTERISK_PHONES_N); $all_vars enthaelt alle
     */
    return array ($VARS, $all_vars, $comments);
}


function removeInactive($vars)
{
    foreach ($vars as $key1 => $var1) {
        if (is_array($var1)) {
            foreach ($var1 as $key2 => $var2) {
                foreach ($var2 as $key3 => $var3) {
                    if ($key3 == 'ACTIVE' && $var3 == 'no') {
                        unset ($vars[$key1][$key2]);
                        break;
                    } elseif (is_array($var3)) {
                        foreach ($var3 as $key4 => $var4) {
                            foreach ($var4 as $key5 => $var5) {
                            }

                            if ($key5 == 'ACTIVE' && $var5 == 'no') {
                                unset ($vars[$key1][$key2][$key3][$key4]);
                                break;
                            } elseif (is_array($var5)) {
                                foreach ($var5 as $key6 => $var6) {
                                    foreach ($var6 as $key7 => $var7) {
                                    }

                                    if ($key7 == 'ACTIVE' && $var7 == 'no') {
                                        unset ($vars[$key1][$key2][$key3][$key4][$key5][$key6]);
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    return $vars;

}


function multi_array_search($search_value, $array1, $keyname = '', $exact = false)
{
    if (isset ($array1)) {
        foreach ($array1 as $key1 => $array2) {
            foreach ($array2 as $key2 => $value) {
                /**
                 * preg_match failed if $value is empty or is Array
                 */
                if (is_array($value) || $value == '' ) {
                    continue;
                }

                if ($exact) {
                    if ($search_value == $value &&
                       (empty ($keyname) || $key2 == $keyname)) {
                        $result[] = $key1;
                    }
                } else {
                    $quoted = preg_quote($search_value);
                    if (preg_match('@'.$quoted.'@', $value) &&
                       (empty ($keyname) ||
                       is_integer(strpos($key2, $keyname)))) {
                        $result[] = $key1;
                    }
                }
            }
        }
    }

    if (!empty($result)) {
        if (is_array($result)) {
            return $result;
        } else {
            return false;
        }
    } else {
        return false;
    }

}


function get_acc_by_varname($type, $string, $varname, $returnType = '', $debug = true)
{
    global $VARS;

    $typeUpper = strtoupper($type);

    if ($string == '') {
        err("Empty searchstring for 'ASTERISK_".$typeUpper."_n_$varname'", __LINE__);
        return false;
    }

    $key = multi_array_search($string, $VARS['ASTERISK_'.$typeUpper], $varname);

    if (empty ($key)) {
        if ($debug)
            err("Could not find '$string' in any 'ASTERISK_".$typeUpper."_n_$varname'", __LINE__);
        return false; # Nix gefunden
    }

    $idx = $key[0];

    if ($returnType == 'array') {
        $accdata = $VARS['ASTERISK_'.$typeUpper][$idx];
        $accdata["key"] = $idx;
    } else
        $accdata = $VARS['ASTERISK_'.$typeUpper][$idx][$varname];

    return $accdata;

}


function write_vars($VARS, $prefix = '', $comments)
{
    $begin_col = 50;

    foreach ($VARS as $key => $value) {
        if (is_array($value)) {
            write_vars($value, $prefix.$key.'_', $comments);
        } else {
            if ($key == 'comment') { // Hier werden die Zeilen ausgegeben, die keine Variabele enthalten (Z.b. Leerzeilen oder Trennlinien).
                echo $value;
                continue;
            }

            if (empty ($key))
                $prefix = substr($prefix, 0, -1);

            $comment = $comments[preg_replace('/[0-9]+/', 'n', $prefix.$key)]; // Bei alle Variablenamen werden alle Zahlen durch "n" ersetzt; Der Kommentar ist dann z.B. $comment["ASTERISK_PHONES_n_MSN"].

            $var_part = $prefix.$key."='".$value."'"; // Die Zeile wird zusammengesetzt

            $spaces = $begin_col - strlen($var_part); // Es wird berechnet, wieviele Leerzeichen eingefuegt werden sollen

            if ($spaces <= 0)
                $spaces = 1; // Wenn die Anzahl der Leerzeichen kleiner 1 ist, werden sie auf 1 gesetzt.

            echo $var_part.str_repeat(' ', $spaces).$comment."\n"; // Ausgabe der Zeile.
        }
    }

}

?>
