<?php
/**
 *---------------------------------------------------------------------------
 * A Script to write an Eisfair-Asterisk configuration file
 *
 * Creation:     2005-10-28 fw
 * Last Update:  $Id: wcExtensions.php 8207 2014-12-17 12:12:00Z holbru $
 *
 * Copyright (c) 2005 Fabian Wolter, fabian(at)eisfair(dot)org
 * Copyright (c) 2014-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------
 */

if (empty($wcContent)) {
    $wcContent = '';
}

if ($VARS["ASTERISK_VBOX_PLAY_INSTRUCTIONS"] != "yes")
    define("VBOX_OPTIONS", "s");
else
    define("VBOX_OPTIONS", "");

if ($VARS["ASTERISK_AREA_CODE"] != "")
    $areaCode = "\${areaCode}";

$errorExten = "include => error\n\n";

if ($VARS["ASTERISK_PHONES_N"] == 0) {
    err("You must have at least one local telephone defined (ASTERISK_PHONES_*). Aborting...");
    exit;
}

$wcContent .= "[general]\n\n";

$wcContent .= "static=yes\n";
$wcContent .= "writeprotect=no\n";

if ($VARS["ASTERISK_LCR"] == "yes")
    $wcContent .= "#include /etc/asterisk/lcr/extensions.tsblcr\n\n";
else
    $wcContent .= "\n";

$wcContent .= "[globals]\n";
$wcContent .= "localdialtimeout=" . DIALTIMEOUT . "\n\n";

$wcContent .= "[default]\n";
$wcContent .= "include => user\n";
$wcContent .= "include => test\n";

if ($VARS['ASTERISK_CALLBACK'] == 'yes') {
    $wcContent .= 'include => callback-in'."\n";
}

$wcContent .= 'include => incoming'."\n";
$wcContent .= 'include => outgoing'."\n";
$wcContent .= $errorExten;

$wcContent .= "[test]\n";

$e = new exten;
$e->setExten('#100');
$e->setCmd('Answer');
$e->setCmd('MP3Player', '/usr/share/asterisk/mohmp3/fpm-calm-river.mp3');
$e->setHangup(true);
$wcContent .= $e->getData();

$e = new exten;
$e->setExten("#101");
$e->setCmd("Answer");
$e->setCmd("Milliwatt");
$e->setHangup(TRUE);
$wcContent .= $e->getData();

$e = new exten;
$e->setExten("#102");
$e->setCmd("Answer");
$e->setCmd("SayDigits", "\${CALLERIDNUM}");
$e->setHangup(TRUE);
$wcContent .= $e->getData();

$e = new exten;
$e->setExten("#103");
$e->setCmd("Answer");
$e->setCmd("Playback", "demo-echotest");
$e->setCmd("Echo");
$e->setCmd("Playback", "demo-echodone");
$e->setHangup(TRUE);
$wcContent .= $e->getData();

$e = new exten;
$e->setExten("#104");
$e->setCmd("Answer");
$e->setCmd("AGI", "date.sh");
$e->setCmd("SendText", "\${ADATE}");
#$e->setCmd("SayUnixTime", ",localtime,A 'digits/at' R");
$e->setCmd("sayPhonetic", "\${ADATE:5}");
$e->setHangup(TRUE);
$wcContent .= $e->getData();

$wcContent .= "[outgoing]\n";
###### Outgoing context

foreach ($VARS["ASTERISK_PHONES"] as $key => $phone) { # Contexte in outgoing inkludieren

    $all_internal_msns[] = array (); # declare var as an array for array_search below

    if (array_search($phone["MSN"], $all_internal_msns) != FALSE) {
        err("The internal MSN " . $phone["MSN"] . " is specified twice in the configuration, but ASTERISK_PHONES_n_MSN has to be unique! Ignoring ASTERISK_PHONES_" . $key . "_MSN.");
        unset ($VARS["ASTERISK_PHONES"][$key]);
    }

    $wcContent .= "include => " . $phone["MSN"] . "\n";

    $all_internal_msns[] = $phone["MSN"]; # Alle internen MSNs fr die Fehlerroutinen sammeln
}

$wcContent .= "\ninclude => incoming\n";

# Bestimmen des Dialprefixes und wieviel abgeschnitten werden soll am anfang der Nummer

getDialprefix("internal");
getDialprefix("sip");
getDialprefix("iax");
getDialprefix("capi");

foreach ($VARS["ASTERISK_PHONES"] as $key => $phone) {

    unset ($fallbackDialCmds);
    unset ($sipfallback);

    $wcContent .= "\n[" . $phone["MSN"] . "]\n";
    $wcContent .= ";;; Phone " . $phone["MSN"] . " ;;;\n";

    ### Outgoing SIP calls
    $wcContent .= writeOutgoing($phone, "sip");

    ### Outgoing IAX calls
    $wcContent .= writeOutgoing($phone, "iax");

    ### Outgoing CAPI/HFCS calls
    $wcContent .= "; Outgoing calls over CAPI\n";

    if (($VARS["ASTERISK_LOAD_CAPI"] == "yes" || $VARS["ASTERISK_LOAD_DAHDI"] == "yes") && $phone["OUTGOING_MSN"] != "") {

        $e = new exten;

        $type = "capi";

        $e->setExten("_" . DIALPREFIX_CAPI . ".");
        $e->setCID($phone["MSN"]);

        if ($VARS["ASTERISK_RECORD_CONVERSATION"] == "yes") {
            $e->setCmd("Monitor", "wav,\${UNIQUEID},bm");
        }

        if ($phone["OUTGOING_HFCS_GROUP"] == "") {
            $pstnTrunkLCR = "CAPI/contr1";
            $pstnTrunkNorm = "CAPI/contr1/";
            $pstnTrunkLast = "/b";
        } else {
            $pstnTrunkLCR = "Dahdi/g" . $phone["OUTGOING_HFCS_GROUP"] . "/";
            $pstnTrunkNorm = $pstnTrunkLCR;
            unset ($pstnTrunkLast);
        }

        if ($VARS["ASTERISK_LCR"] == "yes" && $phone["LCR"] == "yes") {

            $priotmp = $e->prio + 2;

            if ($VARS["ASTERISK_AREA_CODE"] != "") {
                if ($type == "sip")
                    $cut_tmp = DIALPREFIX_SIP;
                elseif ($type == "iax") $cut_tmp = DIALPREFIX_IAX;
                else
                    $cut_tmp = DIALPREFIX_CAPI;

                if ($cut_tmp == "x")
                    $cut_new = ":0";
                else {
                    if ($type == "sip")
                        $cut_new = CUT_SIP;
                    elseif ($type == "iax") $cut_new = CUT_IAX;
                    else
                        $cut_new = CUT_CAPI;
                }

                $e->setCmd("GotoIf", "\$[\"\${EXTEN$cut_new:1}\" = \"0\"]?$priotmp");
                $e->setCmd("Set", "areaCode=" . $VARS["ASTERISK_AREA_CODE"]);
            }

            $e->setCmd("Macro", "tsblcr,\${areaCode}\${EXTEN" . CUT_CAPI . "},$lcrRoutingFile," . $phone["OUTGOING_MSN"] . "," . $phone["OUTGOING_NAME"] . ",$pstnTrunkLCR,$lcrSipConf");
        } else {
            $e->setCmd("Set", "CALLERID(number)=" . $phone["OUTGOING_MSN"]);
            $e->setCID($phone["OUTGOING_MSN"]);
            $e->setCmd("Set", "CALLERID(name)=");
            $e->setCmd("Dial", $pstnTrunkNorm . "\${EXTEN" . CUT_CAPI . "}$pstnTrunkLast," . DIALTIMEOUT . "," . OPTIONS_OUTGOING);
            $e->setHangup();
        }

        $wcContent .= $e->getData();
        $wcContent .= "\n";

    } else {
        $e = new exten;

        $e->setExten("_" . DIALPREFIX_CAPI . ".");
        $e->setCID($phone["MSN"]);
        $e->setCmd("Playback", "ss-noservice");
        $e->setHangup(TRUE);
        $wcContent .= $e->getData();
    }
}

if ($VARS['ASTERISK_CALLTHROUGH_N'] > 0) {
    foreach ($VARS['ASTERISK_CALLTHROUGH'] as $ct) { # ct = callthrough
        $wcContent .= "\n[DISA-" . $ct["MSN"] . "]\n";

        $wcContent .= "include => incoming\n\n";

        ### Outgoing SIP calls

        $wcContent .= writeOutgoingCt($ct, DIALPREFIX_SIP, "sip");

        ### Outgoing IAX calls

        $wcContent .= writeOutgoingCt($ct, DIALPREFIX_IAX, "iax");

        ### Outgoing CAPI calls

        if ($ct["ALLOW_CAPI"] == "yes") {
            $e = new exten;
            $e->setExten("_" . DIALPREFIX_CAPI . ".");

            $wcContent .= "; Outgoing calls over CAPI\n";
            $e->setCmd("Set", "CALLERID(name)=");
            $e->setCmd("Set", "CALLERID(number)=" . $ct["OUTGOING_MSN"]);

            if ($ct["OUTGOING_HFCS_GROUP"] == "")
                $e->setCmd("Dial", "CAPI/contr1/\${EXTEN" . CUT_CAPI . "}/b," . DIALTIMEOUT);
            else
                $e->setCmd("Dial", "Dahdi/g" . getHfcsGroup($ct["OUTGOING_HFCS_GROUP"]) . "/\${EXTEN" . CUT_CAPI . "}," . DIALTIMEOUT);

            $e->setHangup();
            $wcContent .= $e->getData();
        }
    }
}

###### Incoming context

$wcContent .= "\n[incoming]\n";
$wcContent .= "include => default\n\n";

### incoming SIP calls

if (isset ($VARS["ASTERISK_SIP"])) { # Ist z.b. nicht gesetzt, wenn keine SIP-Accounts defniert sind
    $wcContent .= writeIncoming($VARS["ASTERISK_SIP"], "sip");
}

### incoming IAX calls

if (isset ($VARS["ASTERISK_IAX"])) { # Ist z.b. nicht gesetzt, wenn keine SIP-Accounts defniert sind
    $wcContent .= writeIncoming($VARS["ASTERISK_IAX"], "iax");
}

### incoming CAPI calls

if ($VARS["ASTERISK_LOAD_CAPI"] == "yes" || $VARS["ASTERISK_LOAD_DAHDI"] == "yes") {

    list (, $incomingMsns) = getAllMsns();

    $wcContent .= writeIncoming($incomingMsns, "capi");
}

### incoming internal calls

foreach ($VARS["ASTERISK_PHONES"] as $key => $phone) {

    unset ($netbiosNames);

    $e = new exten;

    if (is_file("/var/lib/asterisk/vbox")) {
        $ext = $VARS["ASTERISK_DIALPREFIX_INTERNAL"] . substr($phone["MSN"], 1);
    } else {
        $ext = $VARS["ASTERISK_DIALPREFIX_INTERNAL"] . $phone["MSN"];
    }

    $e->setExten($ext);

    $wcContent .= "; Incoming (internal) calls for number " . $phone["MSN"] . ". Type of phone: " . $phone["TYPE"] . "\n";

    foreach ($VARS["ASTERISK_PHONES"] as $phone2) { # Hier werden alle Telefone durchgegangen, um den CallerIDName korrekt zu setzen
        if ($phone["MSN"] == $phone2["MSN"])
            continue; # man muss sich nicht selbst anzurufen
        $e->setCID($phone2["MSN"]);
        $e->setCmd("Set", "CALLERID(name)=" . $phone2["OUTGOING_NAME"]);
        $e->prio--;
    }

    $e->prio++;

    foreach ($VARS["ASTERISK_PHONES"] as $phone2) { # Hier werden alle Telefone durchgegangen, um die CallerID korrekt zu setzen
        if ($phone["MSN"] == $phone2["MSN"])
            continue; # man braucht sich nicht selbst anzurufen
        $e->setCID($phone2["MSN"]);
        $e->setCmd("Set", "CALLERID(number)=" . $VARS["ASTERISK_DIALPREFIX_INTERNAL"] . $phone2["MSN"]);
        $e->prio--;
    }

    $e->setCID("");

    $e->setCmd("NoOp", "", 2); # 1 ist durch SetLanguage bereits gesetzt.
    $e->setCmd("NoOp", "", 3);

    $e->prio++;

    if (is_file("/var/lib/asterisk/vbox")) {
        $e->setCmd("GotoIf", "\$[\"\${CALLERID(number)}\" = \"$ext\"]?error,h,1");
    }

    foreach ($VARS["ASTERISK_PHONES"] as $phone2) {
        if (empty ($phone2["EXEC_ON_RING"]))
            continue; # Hier sollen nur die EXEC_ON_RING Variablen ausgewertet werden, wo auch etwas drin steht

        $e->setCmd("System", $phone2["EXEC_ON_RING"]);
    }

    if ($phone["VBOX"] == "yes")
        $localdialtimeout = $phone["VBOX_ANSWER_AFTER"];
    else
        $localdialtimeout = DIALTIMEOUT;

    if (!is_file("/var/lib/asterisk/vbox")) {
        $e->setCmd("Dial", getDialCmd($phone["TYPE"], $phone["MSN"], $key) . ",$localdialtimeout");

        $e->setCmd("Goto", "incoming," . $e->exten . "-\${DIALSTATUS},1");

        if ($phone["VBOX"] == "yes") {
            $e->prio = 1;
            $e->setExten($ext . "-NOANSWER");
            $e->setCmd("VoiceMail", VBOX_OPTIONS . "u" . $phone["MSN"]);
            $e->setHangup(TRUE);
            $e->prio = 1;
            $e->setExten($ext . "-CHANUNAVAIL");
            $e->setCmd("VoiceMail", VBOX_OPTIONS . "b" . $phone["MSN"]);
            $e->setHangup(TRUE);
            $e->prio = 1;
            $e->setExten($ext . "-BUSY");
            $e->setCmd("VoiceMail", VBOX_OPTIONS . "b" . $phone["MSN"]);
            $e->setHangup(TRUE);
            $e->prio = 1;
            $e->setExten($ext . "-CONGESTION");
            $e->setCmd("VoiceMail", VBOX_OPTIONS . "b" . $phone["MSN"]);
            $e->setHangup(TRUE);
            $e->prio = 1;
            $e->setExten($ext . "-.");
            $e->setCmd("VoiceMail", VBOX_OPTIONS . "u" . $phone["MSN"]);
            $e->setHangup(TRUE);
        } else {
            $e->setHangup();
        }
    } else {

        $e->setCmd("Ringing", "");

        $e = writeResolveNumbers($e);

        if (!empty ($VARS["ASTERISK_PHONES"][$key]["NETBIOS_NAME"])) # Sammle alle Befehle aus EXEC_ON_RING wenn diese nicht leer ist und die aktuelle $msn in RING_ON_CAPI steht
            $netbiosNames[] = $VARS["ASTERISK_PHONES"][$key]["NETBIOS_NAME"];

        if (isset ($netbiosNames))
            $netbiosNames = array_unique($netbiosNames);

        if (count($netbiosNames) > 0) {
            foreach ($netbiosNames as $netbiosName) {
                $e->setCmd("System", "echo \"Eingehender Anruf von \${CALLERID(number)} \${CALLERID(name)}\" , smbclient -U \"Asterisk Telefonserver\" -M \"$netbiosName\" &");
            }
        }
        $e->setCmd("Wait", $localdialtimeout);
        $e->setCmd("VoiceMail", VBOX_OPTIONS . "u" . $phone["MSN"]);
        $e->setHangup(TRUE);
    }

    $wcContent .= $e->getData();
}

### incoming VBOX calls

if ($VARS["ASTERISK_PHONES"] > 0) {
    if ($VARS["ASTERISK_VBOX_AUTODETECT_CID"] == "yes") {
        $wcContent .= writeVbox(FALSE, TRUE);
    } else {
        foreach ($VARS["ASTERISK_PHONES"] as $key => $phone) {

            if ($phone["VBOX"] != "yes")
                continue;

            if ($phone["VBOX_ANSWER_AFTER"] == "")
                err("ASTERISK_PHONES_" .
                $key . "_VBOX_ANSWER_AFTER is empty! Skipping...", __LINE__);

            $wcContent .= writeVbox($phone["MSN"]);
        }
    }
}

if (isset($VARS['ASTERISK_VBOX_N']) && $VARS['ASTERISK_VBOX_N'] > 0) {
    foreach ($VARS['ASTERISK_VBOX'] as $key => $vbox) {
        if (!empty($vbox['NUMBER'])) {
     //       err('ASTERISK_VBOX_' . $key . '_NUMBER is empty! Skipping...', __LINE__);
     //       continue;
     //   }

            $wcContent .= writeVbox($vbox['NUMBER']);
        }
    }
}

validateVbox("MSN", TRUE);
validateVbox("IAX");
validateVbox("SIP");

### incoming DISA calls
if ($VARS["ASTERISK_PHONES"] > 0) {
    foreach ($VARS["ASTERISK_PHONES"] as $phone) {

        if ($phone["TYPE"] != "disa")
            continue; # Es sollen nur nur DISA Telefone bercksichtigt werden

        $wcContent .= "; Incoming DISA calls for " . $phone["MSN"] . "\n";
        $wcContent .= "exten => " . $VARS["ASTERISK_DIALPREFIX_DISA"] . "/" . $phone["MSN"] . ",1,Answer\n";
        $wcContent .= "exten => " . $VARS["ASTERISK_DIALPREFIX_DISA"] . "/" . $phone["MSN"] . ",3,DISA(no-password,outgoing)\n\n";
    }
}

// incoming DISA callthroughs

/**
 * ct = callthrough
 */
if ($VARS['ASTERISK_CALLTHROUGH_N'] > 0) {
    foreach ($VARS['ASTERISK_CALLTHROUGH'] as $key => $ct) {
        if ($ct['MSN'] == $VARS['ASTERISK_DIALPREFIX_DISA']) {
           err('ASTERISK_CALLTHROUGH_' . $key . '_MSN may not be equal to ASTERISK_DIALPREFIX_DISA!', __LINE__);
        }

        /**
         * Wenn nur bestimmte Caller IDs erlaubt sind, muss
         * fuer jede Caller ID ein Block geschrieben werden,
         * wenn nichts angegeben wurde, wird die ex-girlfriend-logic
         * nicht verwendet. (empty funktioniert hier nicht,
         * da auch 0 TRUE ergibt)
         */
        if ($ct['ALLOW_N'] > 0) {
            foreach ($ct['ALLOW'] as $allow) {

                $e = new exten;
                $e->setExten($ct['MSN'], TRUE);
                $e->setCID($allow['CALLERID']);

                $wcContent .= '; Incoming DISA callthroughs for ';
                $wcContent .= $ct['MSN'] . ' by ' . $allow['CALLERID'] . "\n";

                $e->setCmd('Answer');

                if ($allow['PIN'] != '') {
                    $e->setCmd('Set', 'TIMEOUT(digit)=5');
                    $e->setCmd('Set', 'TIMEOUT(response)=10');
                    $e->setCmd('Authenticate', $allow['PIN']);
                }

                $e->setCmd('DISA', 'no-password,DISA-' . $ct['MSN']);
                $wcContent .= $e->getData();
                $wcContent .= "\n";
            }
        }

        if ($ct['ANONYMOUS_ACCESS'] == 'yes') {

            /**
             * Die PIN darf nicht leer sein, da sonst jeder
             * callthrough machen kann, der die MSN kennt.
             */
            if ($ct['ANONYMOUS_PIN'] == '') {
                warn('ASTERISK_CALLTHROUGH_' . $key . '_ANONYMOUS_PIN may not be empty. Security reasons! Skipping...');
                continue;
            }

            $e = new exten;
            $e->setExten($ct['MSN']);

            $wcContent .= '; Incoming DISA callthroughs for ';
            $wcContent .= $ct['MSN'] . ' by unkown callerids'."\n";

            $e->setCmd('Answer');
            $e->setCmd('Set', 'TIMEOUT(digit)=5');
            $e->setCmd('Set', 'TIMEOUT(response)=10');
            $e->setCmd('Authenticate', $ct['ANONYMOUS_PIN']);
            $e->setCmd('DISA', 'no-password,DISA-' . $ct['MSN']);
            $wcContent .= $e->getData();
        }
    }
}

$wcContent .= "\n";

if (isset($VARS['ASTERISK_IVR']) && $VARS['ASTERISK_IVR'] > 0) {
    foreach ($VARS['ASTERISK_IVR'] as $key => $ivr) {

        if (!empty($ivr['NUMBER'])) {

            $wcContent .= "; Context call for IVR menu $key\n";

            $e = new exten;
            $e->setExten($ivr['NUMBER']);
            $e->setCmd('Goto', "IVR$key,s,1");
            $wcContent .= $e->getData();

            $wcContent .= "\n";
        }
    }
}
$wcContent .= writeRedirectionContextHelper();

if (isset($VARS['ASTERISK_IVR']) && $VARS['ASTERISK_IVR'] > 0) {
    foreach ($VARS['ASTERISK_IVR'] as $key => $ivr) {

        if (!empty($ivr['NUMBER'])) {
            $wcContent .= writeIvr($ivr, $key);
        }
    }
}

### Call back

if ($VARS["ASTERISK_CALLBACK"] == "yes") {
    $type = $VARS["ASTERISK_CALLBACK_OUTGOING_TYPE"];
    $from = $VARS["ASTERISK_CALLBACK_OUTGOING_FROM"];

    $wcContent .= "[callback-in]\n";
    $wcContent .= "exten => " . $VARS["ASTERISK_CALLBACK_LISTEN_NUMBER"] . ",1,GotoIf(\$[\"\${CALLERID(num)}\" = \"Anonymous\"]?callback-withoutcallid,s,1:callback-withcallid,s,1)\n";

    $wcContent .= "\n[callback-withoutcallid]\n";

    $e = new exten;
    $e->setExten("s");
    $e->setCmd("Answer");
    $e->setCmd("Wait", "1");

    $wcContent .= "\n; EingabeSchleife zur Zieleingabe\n";
    $e->setCmd("Set", "MYNR=");
    $prio[0] = $e->prio;
    $e->setCmd("Background", "privacy-prompt");
    $e->setCmd("Set", "TIMEOUT(response)=10");
    $prio[1] = $e->prio;
    $e->setCmd("WaitExten");
    $wcContent .= $e->getData();

    $e = new exten;
    $e->setExten("_X");
    $e->setCmd("Set", "MYNR=\${MYNR}\${EXTEN}");
    $e->setCmd("Goto", "s," . $prio[1]);
    $wcContent .= $e->getData();

    $wcContent .= "\n; Zuruecksetzen\n";
    $e = new exten;
    $e->setExten("*");
    $e->setCmd("Goto", "s," . $prio[0]);
    $wcContent .= $e->getData();

    $wcContent .= "\n; Eigene Rufnummer uebergeben\n";
    $e = new exten;
    $e->setExten("#");
    $e->setCmd("Background", "queue-thankyou");
    $e->setCmd("Set", "CALLERID()=\${MYNR}");
    $e->setCmd("AGI", "callback.sh");
    $e->setHangup(TRUE);
    $wcContent .= $e->getData();

    $wcContent .= "\n; Handling fuer Timeout & bye\n";
    $e = new exten;
    $e->setExten("t");
    $e->setCmd("Goto", "#,1");
    $wcContent .= $e->getData();

    $wcContent .= "\n[callback-withcallid]\n";
    $e = new exten;
    $e->setExten("s");
    $e->setCmd("AGI", "callback.sh");
    $e->setCmd("Set", "PRI_CAUSE=17");
    $e->setCmd("Hangup", ""); # Hier muss sofort Hangup verwendet werden, da nur dann PRI_CAUSE wirksam wird (Congestion kommt niemals zum Hangup)
    $wcContent .= $e->getData();

    $wcContent .= "\n;;;;;;;;;;;\n\n";

    $wcContent .= "[callback-dial]\n";
    $e = new exten;
    $e->setExten("_X.");

    if ($type == "capi" || preg_match("/^hfcs/", $type)) {
        $fromString = "";
        if ($VARS["ASTERISK_LCR"] == "yes") {
            if ($type == "capi") {
                $pstnTrunkLCR = "CAPI";
            } else {
                $pstnTrunkLCR = "Dahdi/g" . getHfcsGroup($type);
            }
            $e->setCmd("Macro", "tsblcr,\${NR},$lcrRoutingFile,$from,,$pstnTrunkLCR,$lcrSipConf");
        } else {
            $e->setCmd("Set", "CALLERID(num)=$from");
        }
    } else
        $fromString = $from;

    $dialCmd = getDialCmd($type, "\${EXTEN}", "", "", $fromString);

    if (!$dialCmd)
        err("Invalid value(s) in ASTERISK_CALLBACK_OUTGOING_TYPE and/or ASTERISK_CALLBACK_OUTGOING_FROM", __LINE__);

    $e->setCmd("Dial", $dialCmd . ",\${localdialtimeout}");

    $e->setCmd("Hangup", "");
    $wcContent .= $e->getData();

    $wcContent .= "\n[callback-go]\n";
    $e = new exten;
    $e->setExten("s");

    $e->setCmd("Set", "TIMEOUT(response)=10");
    $e->setCmd("GotoIf", "$[\"\${password}\" = \"\"]?" . ($e->prio + 2));
    $e->setCmd("Authenticate", "$\{password}");

    $prio[2] = $e->prio;
    $e->setCmd("Set", "NR=");
    $e->setCmd("Background", "vm-enter-num-to-call");
    $e->setCmd("Set", "TIMEOUT(response)=10");
    $prio[3] = $e->prio;
    $e->setCmd("WaitExten", "");
    $wcContent .= $e->getData();

    $e = new exten;
    $e->setExten("_X");
    $e->setCmd("Set", "NR=\${NR}\${EXTEN}");
    $e->setCmd("Goto", "s," . $prio[3]);
    $wcContent .= $e->getData();

    $e = new exten;
    $e->setExten("*");
    $e->setCmd("Goto", "s," . $prio[2]);
    $wcContent .= $e->getData();

    $e = new exten;
    $e->setExten("#");

    if ($type == "capi" || preg_match("/^hfcs/", $type)) {
        $fromString = "";
        if ($VARS["ASTERISK_LCR"] == "yes") {
            if ($type == "capi") {
                $pstnTrunkLCR = "CAPI";
            } else {
                $pstnTrunkLCR = "Dahdi/g" . getHfcsGroup($type);
            }
            $e->setCmd("Macro", "tsblcr,\${NR},$lcrRoutingFile,$from,,$pstnTrunkLCR,$lcrSipConf");
        } else {
            $e->setCmd("Set", "CALLERID(num)=$from");
        }
    } else
        $fromString = $from;

    if (!$dialCmd)
        err("Invalid value(s) in ASTERISK_CALLBACK_OUTGOING_TYPE and/or ASTERISK_CALLBACK_OUTGOING_FROM", __LINE__);
    elseif ($VARS["ASTERISK_LCR"] != "yes") {
        $dialCmd = getDialCmd($type, "\${NR}", "", $fromString);
    }

    $e->setCmd("Dial", $dialCmd . ",\${localdialtimeout}");
    $prio[4] = $e->prio;
    $e->setCmd("Goto", "#-\${DIALSTATUS},1");
    $wcContent .= $e->getData();

    $e = new exten;
    $e->setExten("#-NOANSWER");
    $e->setCmd("Playback", "vm-nobodyavail");
    $e->setCmd("Goto", "s," . $prio[2]);
    $wcContent .= $e->getData();

    $e = new exten;
    $e->setExten("#-BUSY");
    $e->setCmd("Busy", "");
    $wcContent .= $e->getData();

    $e = new exten;
    $e->setExten("#-.");
    $e->setCmd("Playback", "tt-somethingwrong");
    $e->setHangup(TRUE);
    $wcContent .= $e->getData();

    $wcContent .= "\n; Handling fuer Timeout\n";
    $e = new exten;
    $e->setExten("t");
    $e->setCmd("GotoIf", "\$[\"\${NR}\" != \"\"]?#,1");
    $wcContent .= $e->getData() . "\n";
}

### Channel status handling

$wcContent .= "[dialStates]\n";
$e = new exten;
$e->setExten("NOANSWER");
$e->setCmd("Playback", "vm-nobodyavail");
$e->setHangup(TRUE);
$wcContent .= $e->getData();

$e = new exten;
$e->setExten("BUSY");
$e->setCmd("Busy", "");
$e->setHangup(TRUE);
$wcContent .= $e->getData();

$e = new exten;
$e->setExten("CHANUNAVAIL");
$e->setCmd("Congestion", "");
$e->setHangup(TRUE);
$wcContent .= $e->getData();

$e = new exten;
$e->setExten("CONGESTION");
$e->setCmd("Congestion", "");
$e->setHangup(TRUE);
$wcContent .= $e->getData();

### Error handling

$wcContent .= "\n[error]\n";
//if (is_file("/var/lib/asterisk/vbox")) {
//    $wcContent .= "exten => h,1,NoOp\n";
//} else {
$wcContent .= "exten => h,1,Hangup\n";
//}
$e = new exten;

$e->setExten("_[*#X].");

if ($VARS["ASTERISK_ADVANCED_ERROR_MSG"] == "yes")
    $e->setCmd("Playback", "error");

$e->setCmd("Playback", "pbx-invalid");
$e->setHangup(TRUE);
$wcContent .= $e->getData();

$wcContent .= "[user]\n";

$wcContent .= shell_exec('for i in `ls /var/install/config.d/asterisk_extensions.*.sh 2> /dev/null`; do echo ";;; $i"; sh $i; done');
?>
