<?php
/**
 *---------------------------------------------------------------------------
 * A Script to write an eisfair-Asterisk configuration file
 *
 * Creation:     2015-11-07 StSa
 * Last Update:  $Id: wcSCCP.php 8115 2014-11-20 22:40:48Z holbru $
 *
 * Copyright (c) 2015-2019 Stephan Sanders, stephan@sanders.koeln 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------
 */

if (empty($wcContent)) {
    $wcContent = '';
}

// ******************************************************************
//            Global Part
// ******************************************************************

$wcContent.= '[general]'."\n";
$wcContent.= 'servername = Asterisk'."\n";
$wcContent.= 'keepalive = 60'."\n";
$wcContent.= 'debug = 1'."\n";
$wcContent.= 'context = default'."\n";
$wcContent.= 'dateformat = D.M.Y'."\n";
$wcContent.= 'bindaddr = '.$VARS['ASTERISK_LISTEN_ADDR']."\n";
$wcContent.= 'port = 2000'."\n";

//$wcContent .= writeCodecs();
// Not all CiscoPhone or sccp Module doesn't support all Codecs or best selection! 
// So we use static settings

$wcContent.= 'disallow = all'."\n";
//$wcContent.= ';allow = g722'."\n";
$wcContent.= 'allow = alaw'."\n";
$wcContent.= 'allow = ulaw'."\n";
$wcContent.= 'allow = g729'."\n";

$wcContent.= 'firstdigittimeout = 16'."\n";
$wcContent.= 'digittimeout = 8'."\n";
$wcContent.= 'autoanswer_ring_time = 1'."\n";
$wcContent.= 'autoanswer_tone = 0x32'."\n";
$wcContent.= 'remotehangup_tone = 0x32'."\n";
$wcContent.= 'transfer_tone = 0'."\n";
$wcContent.= 'transfer_on_hangup = off'."\n";
$wcContent.= 'callwaiting_tone = 0x2d'."\n";
$wcContent.= 'musicclass=default'."\n";
$wcContent.= 'language='.$VARS['ASTERISK_LANGUAGE']."\n";

// $wcContent.= ';permit=192.168.1.0/255.255.255.0'."\n";
// Hope this format will be work too!
if (isset ($VARS['ASTERISK_LOCALNETS'])) {
    foreach ($VARS['ASTERISK_LOCALNETS'] as $localnet) {
        $wcContent .= 'permit='.$localnet['IP']."\n";
    }
}

$wcContent.= 'dnd = on'."\n";
$wcContent.= 'sccp_tos = 0x68'."\n";
$wcContent.= 'sccp_cos = 4'."\n";
$wcContent.= 'audio_tos = 0xB8'."\n";
$wcContent.= 'audio_cos = 6'."\n";
$wcContent.= 'video_tos = 0x88'."\n";
$wcContent.= 'video_cos = 5'."\n";
$wcContent.= 'echocancel = on'."\n";
$wcContent.= 'silencesuppression = off'."\n";
$wcContent.= 'private = on'."\n";
$wcContent.= 'callanswerorder=oldestfirst'."\n";
$wcContent.= 'directed_pickup_modeanswer = on'."\n";
$wcContent.= 'hotline_enabled=no        ; can devices without configuration register'."\n";
$wcContent.= 'hotline_context=default   ; context for hotline'."\n";
$wcContent.= 'hotline_extension=10      ; extension will be dialed on offHook'."\n\n";


// ******************************************************************
//                  per Device
// ******************************************************************


if (isset ($VARS['ASTERISK_PHONES']))
    foreach ($VARS['ASTERISK_PHONES'] as $phone) {
        if ($phone['TYPE'] != 'sccp') {
            continue;
        }

        $wcContent.= '[SEP'.$phone['MAC_ADDR']."]\n";
        $wcContent.= 'type = device'."\n";
        $wcContent.= 'devicetype = 7961                         ;Ciscomodel e.g. 7961'."\n";
        $wcContent.= 'dtmfmode = auto'."\n";
        $wcContent.= ';imageversion=SCCP41.9-4-2SR1-1S'."       ;Only needed for FW upgrades\n";
        $wcContent.= 'device => SEP'.$phone['MAC_ADDR']."\n";
        $wcContent.= 'description = '.$phone['OUTGOING_NAME']."\n";
        $wcContent.= 'tzoffset    = 0'."\n";
        $wcContent.= 'button = line, '.$phone['MSN'].',default'."\n";
        $wcContent.= 'button = line, 11             ;second MSN/Extension'."\n";
        $wcContent.= 'button = feature,Record calls,monitor'."\n";
        $wcContent.= 'button = speeddial,Helpline, 023451234'."\n";
        $wcContent.= 'button = feature,Anonym,privacy,callpresent'."\n";
//    Anleitung Anrufliste suchen!
//         $wcContent.= 'button = feature,Anonym,ANRUFLISTE'."\n";
        $wcContent.= 'transfer = on'."\n";
        $wcContent.= 'park = on'."\n";
        $wcContent.= 'cfwdall = on'."\n";
        $wcContent.= 'cfwdbusy = on'."\n";
        $wcContent.= 'cfwdnoanswer = on'."\n";
        $wcContent.= 'directed_pickup = on'."\n";
        $wcContent.= 'directed_pickup_context = default'."\n";
        $wcContent.= 'directed_pickup_modeanswer = on'."\n";
        $wcContent.= 'mwilamp = on'."\n";
        $wcContent.= 'mwioncall = off'."\n\n";

    }

// ******************************************************************
//                  per Line
// ******************************************************************

if (isset ($VARS['ASTERISK_PHONES']))
    foreach ($VARS['ASTERISK_PHONES'] as $phone) {
        if ($phone['TYPE'] != 'sccp') {
            continue;
        }

        $wcContent.= '['.$phone['MSN']."]\n";
        $wcContent.= 'id = '.$phone['MSN']."\n";
        $wcContent.= 'type = line'."\n";
        $wcContent.= 'pin = 4356'."\n";
        $wcContent.= 'label = Line/MSN '.$phone['MSN']."\n";
        $wcContent.= 'description = Line/MSN '.$phone['MSN']."\n";
        $wcContent.= 'mailbox = 10'."\n";
        $wcContent.= 'cid_name = '.$phone['OUTGOING_NAME']."\n";
        $wcContent.= 'cid_num = '.$phone['MSN']."\n";
        $wcContent.= 'accountcode=79002'."\n";
        $wcContent.= 'callgroup=1,4-9'."\n";
        $wcContent.= 'pickupgroup=1,3-9'."\n";
        $wcContent.= 'context = outgoing'."\n";
        $wcContent.= 'incominglimit = 2'."\n";
        $wcContent.= 'transfer = on'."\n";
        $wcContent.= 'vmnum = 600'."\n";
        $wcContent.= 'trnsfvm = 1000'."\n";
        $wcContent.= 'secondary_dialtone_digits = 9'."\n";
        $wcContent.= 'secondary_dialtone_tone = 0x22'."\n";
        $wcContent.= 'musicclass=default'."\n";
        $wcContent.= 'language='.$VARS['ASTERISK_LANGUAGE']."\n";
        $wcContent.= 'echocancel = on'."\n";
        $wcContent.= 'silencesuppression = off'."\n";
        $wcContent.= 'dnd = silent'."\n\n";
   }

?>
