<?php
/**
 *---------------------------------------------------------------------------
 * A Script to write an Eisfair-Asterisk configuration file
 *
 * Creation:     2005-10-28 fw
 * Last Update:  $Id: wcSip.php 8428 2015-03-08 22:18:56Z holbru $
 *
 * Copyright (c) 2005 Fabian Wolter, fabian(at)eisfair(dot)org
 * Copyright (c) 2014-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------
 */

if (empty($wcContent)) {
    $wcContent = '';
}

$wcContent .= '[general]'."\n";
//$wcContent .= 'bindport=5060'."\n";
$wcContent .= 'bindport='.$VARS['ASTERISK_SIP_PORT']."\n";
$wcContent .= 'bindaddr=0.0.0.0'."\n";
$wcContent .= 'srvlookup=yes'."\n";
$wcContent .= 'context=default'."\n";
//$wcContent .= "fromdomain=".$VARS["ASTERISK_LISTEN_ADDR"]."\n";
$wcContent .= 'defaultexpirey=300'."\n";
$wcContent .= 'maxexpirey=3600'."\n";

$wcContent .= writeCodecs();

//$wcContent .= 'localnet='.$VARS['ASTERISK_LOCALNET']."\n";
if (isset ($VARS['ASTERISK_LOCALNETS'])) {
    foreach ($VARS['ASTERISK_LOCALNETS'] as $localnet) {
        $wcContent .= 'localnet='.$localnet['IP']."\n";
    }
}

$wcContent .= 'canreinvite=yes'."\n";
$wcContent .= 'tos=reliability'."\n";

if (!empty ($VARS['ASTERISK_EXTERNIP'])) {
    if (preg_match('/[0-255]\.[0-255]\.[0-255]\.[0-255]/', $VARS['ASTERISK_EXTERNIP']))
        $wcContent .= 'externip='.$VARS['ASTERISK_EXTERNIP']."\n";
    else
        $wcContent .= 'externhost='.$VARS['ASTERISK_EXTERNIP']."\n";
}

$wcContent .= 'nat=yes'."\n";
$wcContent .= 'dtmfmode=auto'."\n";
//$wcContent .= 'qualify=no'."\n";
$wcContent .= 'qualify='.$VARS['ASTERISK_SIP_QUALIFY']."\n";
$wcContent .= 'language='.$VARS['ASTERISK_LANGUAGE']."\n\n";

if (isset ($VARS['ASTERISK_SIP'])) {

    $wcContent .= writeIaxSipRegistries('sip');

    $wcContent .= "\n".'; Write SIP-Proxy accounts'."\n";

    foreach ($VARS['ASTERISK_SIP'] as $sip) {

        if (is_integer(strpos($sip['NAME'], 't-online')) && is_integer(strpos($sip['USER'], ':'))) {
            list ($fromuser, $username) = explode(':', $sip['USER']);
        } else {
            $fromuser = $sip['USER'];
            $username = $fromuser;
        }

        $wcContent .= '['.$sip['NAME'].']'."\n";
        $wcContent .= 'type=friend'."\n";
        $wcContent .= 'host='.$sip['HOST']."\n";
        $wcContent .= 'fromuser='.$fromuser."\n";
        $wcContent .= 'defaultuser='.$username."\n";
        $wcContent .= 'secret='.$sip['PASSWORD']."\n";
        $wcContent .= 'fromdomain='.$sip['HOST']."\n";
        $wcContent .= 'context=incoming'."\n";
        $wcContent .= 'insecure=invite'."\n";
        $wcContent .= writeCodecs();
        $wcContent .= "\n\n";

    }
}

$wcContent .= '; Write SIP-Client accounts'."\n";

if (isset ($VARS['ASTERISK_PHONES'])) {
    foreach ($VARS['ASTERISK_PHONES'] as $phone) {

        if ($phone['TYPE'] != 'sip') {
            /**
             * Hier sollen nur SIP-Clients geschrieben werden
             */
            continue;
        }

        $wcContent .= '['.$phone['MSN'].']'."\n";
        $wcContent .= 'type=friend'."\n";
        $wcContent .= 'secret='.$phone['PIN']."\n";
        #$wcContent.= "auth=md5\n";
        $wcContent .= 'host=dynamic'."\n";
        $wcContent .= 'reinvite=no'."\n";
        $wcContent .= 'canreinvite=no'."\n";
        $wcContent .= 'qualify=1000'."\n";
        $wcContent .= 'dtmfmode=auto'."\n";
        $wcContent .= writeCodecs();
        $wcContent .= 'context=outgoing'."\n";

        if ($phone['VBOX'] == 'yes') {
            $wcContent .= 'vmexten='.$VARS['ASTERISK_DIALPREFIX_VBOX'].$phone['MSN']."\n";
            $wcContent .= 'mailbox='.$phone['MSN']."\n";
        }

        if ($VARS['ASTERISK_VBOX_N'] > 0) {
  //       if (count($VARS['ASTERISK_VBOX']) > 0) {
            foreach ($VARS['ASTERISK_VBOX'] as $vbox) {
                if (isset ($vbox['NOTIFY'])) {
                    foreach ($vbox['NOTIFY'] as $notify) {
                        if ($notify['PHONE'] == $phone['MSN'])
                            $wcContent .= 'mailbox='.$vbox['NUMBER']."\n";
                    }
                }
            }
        }
    }
}
?>
