<?php
/**
 *---------------------------------------------------------------------------
 * /var/lib/asterisk/config/writeFunc.php - Contains helper function which are needed in writeContexts.php
 *
 * Creation:     2005-10-28 fw
 * Last Update:  $Id: writeFunc.php 9283 2016-04-14 16:59:35Z holbru $
 *
 * Copyright (c) 2005 Fabian Wolter, fabian(at)eisfair(dot)org
 * Copyright (c) 2014-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------
 */

class exten {
    public $cid;
    public $exten;
    public $block;
    public $prio = 1;
    private $data;

    function setCID($inputCID) {
        $this->cid = $inputCID;
    }

    function setExten($inputExten, $nolanguage = FALSE) {
        global $VARS;

        $this->exten = $inputExten;

        if (!$nolanguage)
            $this->setCmd('Set', 'CHANNEL(language)='.$VARS['ASTERISK_LANGUAGE']);
    }

    function setCmd($cmd, $params = "", $inputPrio = "") {
        if ($this->cid == "")
            $slash = '';
        else
            $slash = '/';

        if (empty ($inputPrio))
            $prio = $this->prio;
        else
            $prio = $inputPrio;

        $this->data .= 'exten => '.$this->exten.$slash.$this->cid.",$prio,$cmd($params)\n";

        if (empty ($inputPrio))
            $this->prio++;
    }

    function setHangup($hangup = FALSE) {
        if ($hangup) {
            $this->setCmd('Congestion', '');
            $this->setCmd('Wait', 10);
            $this->setCmd('Hangup');
            /**
             * Benoetig fuer fallback
             */
            $this->prio -= 3;
        } else {
            $this->setCmd('Goto', "dialStates,\${DIALSTATUS},1");
            /**
             * Benoetig fuer fallback
             */
            $this->prio -= 1;
        }

        $this->data .= "\n";
    }

    function getData() {
        return $this->data;
    }
}

function writeConfigfile($filename, $content) {
    $fp = fopen(CONFIGFILE_PATH.$filename, 'w');
    if ($fp) {
        fwrite($fp, $content);
        fclose($fp);
    } else
        err("Couldn't open ".CONFIGFILE_PATH.$filename." for writing!", __LINE__);
}

function writeConfig($name) {
    global $VARS, $lcrRoutingFile, $lcrSipConf;

    /**
     * Hier wird $wcContent erzeugt
     */
    include 'wc'.$name.'.php';
    writeConfigfile(strtolower($name).'.conf', $wcContent);
}

function writeResolveNumbers($e) {
    global $VARS;

    /**
     * Wenn diese Var auf yes steht soll ueber getName.php per
     * Inverssuche der Name des Anrufers aufgeloest werden
     */
    if ($VARS["ASTERISK_RESOLVE_NUMBERS"] == "yes") {
        $params = '';

        if ($VARS["ASTERISK_RESOLVE_NUMBERS_REMOTE"] == "no")
            $params .= ",local=yes";

        if ($VARS["ASTERISK_RESOLVE_NUMBERS_SOURCE"] == "db") {
            $params .= ",host=".$VARS["ASTERISK_RESOLVE_NUMBERS_DB_HOST"].",user=".$VARS["ASTERISK_RESOLVE_NUMBERS_DB_USER"].",password=".$VARS["ASTERISK_RESOLVE_NUMBERS_DB_PASS"].",dbname=".$VARS["ASTERISK_RESOLVE_NUMBERS_DB_NAME"].",table=".$VARS["ASTERISK_RESOLVE_NUMBERS_DB_TABLE"];
        }
        elseif ($VARS["ASTERISK_RESOLVE_NUMBERS_SOURCE"] == "file") {
            $params .= ",path=".$VARS["ASTERISK_RESOLVE_NUMBERS_PATH"];
        }

        $e->setCmd("AGI", "getName.php$params");
        $e->setCmd("Set", "CALLERID(number)=\${CID}");
        $e->setCmd("Set", "CALLERID(name)=\${NAME}");
    }

    return $e;
}

function writeCodecs() {
    global $VARS;

    $return = "disallow=all\n";

    if (!empty ($VARS["ASTERISK_CODECS"])) {
        $codecs = explode(" ", $VARS["ASTERISK_CODECS"]);

        foreach ($codecs as $codec) {
            $return .= "allow=$codec\n";
        }
    }

    return $return;
}

function writeIaxSipRegistries($type) {
    global $VARS;

    $typeUpper = strtoupper($type);

    if (isset ($VARS["ASTERISK_".$typeUpper])) {
        $return = "; Write Registries\n";

        foreach ($VARS["ASTERISK_".$typeUpper] as $acc) {
            if (is_integer(strpos($acc["NAME"], "t-online")) && is_integer(strpos($acc["USER"], ":"))) {
                list ($tonlNummer, $email) = explode(":", $acc["USER"]);
                $auth = $tonlNummer.":".$acc["PASSWORD"].":".$email;
            } else {
                $auth = $acc["USER"].":".$acc["PASSWORD"];
                if (is_integer(strpos($acc["NAME"], "t-online")))
                    err("name", __LINE__);
                if (is_integer(strpos($acc["USER"], ":")))
                    err("user", __LINE__);
            }
            $return .= "register => ".$auth."@".$acc["HOST"];

            if ($type == "sip") {
                $return .= "/".$acc["CALLERID"]."\n";
            } else {
                $return .= "\n";
            }
        }
        return $return;
    }
}

class phone {

    var $type, $msn, $key;

    function phone($key) {
        global $VARS;

        if (empty ($key)) {
            msg("Key is not set in class <em>phone</em>;error");
            return FALSE;
        }

        $this->type = $VARS["ASTERISK_PHONES"][$key]["TYPE"];
        $this->msn = $VARS["ASTERISK_PHONES"][$key]["MSN"];
        $this->key = $key;
    }

    function getDialCmd() {
        return $this->getChannel().$this->getAddr();
    }

    function getRingOn($type) {
        global $VARS;
        return explode(" ", $VARS["ASTERISK_PHONES"][$this->key]["RING_ON_".strtoupper($type)]);
    }

    function getAddr() {
        if ($this->type == "disa" || $this->type == "capi") {
            $addr = $this->msn;
        }
        elseif (strstr($this->type, "hfcs") !== FALSE) {
            $hfcsGroup = getHfcsGroup($this->type);
            $addr = $this->msn;
        }
        elseif ($this->type == "sip") {
            $addr = $this->msn.$acc;
        }
        elseif ($this->type == "iax") {
            $addr = $this->msn.$acc;
        }
        elseif ($this->type == "skinny") {
            $addr = $this->msn."@cisco".$this->msn;
        }
        elseif ($this->type == "sccp") {
            $addr = $this->msn."@cisco".$this->msn;
        }
        elseif ($this->type == "unistim") {
            $addr = $this->msn."@unistim".$this->msn;
        } else
            msg("ASTERISK_PHONES_".$this->key."_TYPE='".$this->type."' is invalid!;error\n");

        return $addr;
    }

    function getChannel() {
        if ($this->type == "disa" || $this->type == "capi") {
            $channel = "CAPI/contr1/";
        }
        elseif (preg_match("/hfcs/", $this->type)) {
            $hfcsGroup = getHfcsGroup($this->type);
            $channel = "Dahdi/g$hfcsGroup/";
        }
        elseif ($this->type == "sip") {
            $channel = "SIP/";
        }
        elseif ($this->type == "iax") {
            $channel = "IAX/";
        }
        elseif ($this->type == "skinny") {
            $channel = "Skinny/";
        }
        elseif ($this->type == "sccp") {
            $channel = "SCCP/";
        }
        elseif ($this->type == "unistim") {
            $channel = "USTM/";
        } else
            msg("ASTERISK_PHONES_".$this->key."_TYPE='".$this->type."' is invalid!;error\n");

        return $channel;
    }

    function getHfcsGroup() {
        $str = trim($this->type);

        if ($str == "hfcs")
            return 1;

        preg_match("/hfcs([0-9]+)/", $str, $matches);

        if (is_numeric($matches[1]))
            return $matches[1];
        else
            return FALSE;
    }

    function getMsn() {
        return $this->msn;
    }

    function getType() {
        return $this->type;
    }
}

function getDialCmd($type, $msn, $key = "", $localdialtimeout = "", $acc = "") {
    if ($localdialtimeout != "")
        $dialtimeout = $localdialtimeout;
    else
        $dialtimeout = DIALTIMEOUT;

    #$options = ",$dialtimeout,$optionsIncoming$optionsOutgoing";
    $options = '';

    if (!empty ($acc)) {
        $acc = "@$acc";
    }

    if ($type == "disa" || $type == "capi") {
        $cmd = "CAPI/contr1/".$msn."/b".$options;
    }
    elseif (preg_match("/hfcs/", $type)) {
        $hfcsGroup = getHfcsGroup($type);
        $cmd = "Dahdi/g$hfcsGroup/".$msn.$options;
    }
    elseif ($type == "sip") {
        $cmd = "SIP/".$msn.$acc.$options;
    }
    elseif ($type == "iax") {
        $cmd = "IAX/".$msn.$acc.$options;
    }
    elseif ($type == "skinny") {
        $cmd = "Skinny/".$msn."@cisco".$msn.$options;
    }
    elseif ($type == "sccp") {
        $cmd = "SCCP/".$msn.$options;
    }
    elseif ($type == "unistim") {
        $cmd = "USTM/".$msn."@unistim".$msn.$options;
    }
    elseif($type == "internal") {
        $cmd = "Local/".DIALPREFIX_INTERNAL.$msn."@incoming";
    }
    elseif (!empty ($key)) {
        err("ASTERISK_PHONES_".$key."_TYPE='$type' is invalid!", __LINE__);
        return FALSE;
    } else
        return FALSE;

    return $cmd;
}

function getHfcsGroup($str) {
    $str = trim($str);

    if ($str == "hfcs")
        return 1;

    preg_match("/hfcs([0-9]+)/", $str, $matches);

    if (is_numeric($matches[1]))
        return $matches[1];
    else
        return FALSE;
}

function getDialprefix($prefix) {
    global $VARS;

    $prefixUpper = strtoupper($prefix);

    if ($VARS["ASTERISK_DIALPREFIX_".$prefixUpper] == "") { # Wenn der Dialprefix leer ist, soll "x" eingefgt werden
        define("DIALPREFIX_".$prefixUpper, "x");
        define("CUT_".$prefixUpper, "");
    } else {
        define("DIALPREFIX_".$prefixUpper, $VARS["ASTERISK_DIALPREFIX_".$prefixUpper]);
        define("CUT_".$prefixUpper, ":".strlen($VARS["ASTERISK_DIALPREFIX_".$prefixUpper]));
    }
}

function removeEmptyValues($array) {
    foreach ($array as $key => $value) {
        if ($value === "")
            unset ($array[$key]);
    }

    return $array;
}

function getAllMsns() {
    global $VARS;

    if (isset ($VARS["ASTERISK_SIP"])) {
        foreach ($VARS["ASTERISK_SIP"] as $sip) {

            $sipfallback = $sip['FALLBACK'];

            if (!empty ($sipfallback)) {

                $fallbacks = explode(' ', $sipfallback);

                foreach ($fallbacks as $fallback) {
                    list ($type, $outgoing) = explode(':', $fallback);

                    if ($type == 'capi')
                        $all_outgoing_msns[] = $outgoing;
                }
            }

            if (isset($sip['REDIRECT']) && $sip['REDIRECT'] == 'yes') {
                $all_outgoing_msns[] = $sip['REDIRECT_OUTGOING_MSN'];
            }
        }
    }

    foreach ($VARS["ASTERISK_PHONES"] as $phone) {
        $all_outgoing_msns[] = $phone["OUTGOING_MSN"];
        $all_incoming_msns[] = $phone["RING_ON_CAPI"];

        if ($phone["TYPE"] == "disa") {
            $all_incoming_msns[] = $VARS["ASTERISK_DIALPREFIX_DISA"];

            if (is_file("/var/lib/asterisk/vbox"))
                $all_incoming_msns[] = $VARS["ASTERISK_DIALPREFIX_INTERNAL"].substr($phone["MSN"], 1);
            else
                $all_incoming_msns[] = $phone["MSN"];

            if ($VARS["ASTERISK_VBOX_AUTODETECT_CID"] == "yes")
                $all_incoming_msns[] = $VARS["ASTERISK_DIALPREFIX_VBOX"];
        }
    }

    if ($VARS["ASTERISK_MSN_N"] != 0) {

        foreach ($VARS["ASTERISK_MSN"] as $key => $extra) {
            $all_incoming_msns[] = $extra["NUMBER"];
        }
    }

    if ($VARS["ASTERISK_CALLTHROUGH_N"] != 0) {

        /**
         * ct = callthrough
         */
        foreach ($VARS["ASTERISK_CALLTHROUGH"] as $ct) {
            $all_outgoing_msns[] = $ct["OUTGOING_MSN"];
            $all_incoming_msns[] = $ct["MSN"];
        }
    }

    if ($VARS["ASTERISK_REDIRECT_N"] != 0) {
        foreach ($VARS["ASTERISK_REDIRECT"] as $red) {
            if (is_array($red)) {
                foreach ($red["SCHEME"] as $scheme) {
                    if (is_array($scheme)) {
                        foreach ($scheme["TARGET"] as $target) {
                            if ($target["TYPE"] == "capi") {
                                $all_outgoing_msns[] = $target["FROM"];
                            }
                        }
                    }
                }
            }
        }
    }

    if ($VARS["ASTERISK_CALLBACK_LISTEN_NUMBER"] != "" && $VARS["ASTERISK_CALLBACK"] == "yes")
        $all_incoming_msns[] = $VARS["ASTERISK_CALLBACK_LISTEN_NUMBER"];

    if ($VARS["ASTERISK_CALLBACK_OUTGOING_FROM"] != "" && $VARS["ASTERISK_CALLBACK_OUTGOING_TYPE"] == "capi" && $VARS["ASTERISK_CALLBACK"] == "yes")
        $all_outgoing_msns[] = $VARS["ASTERISK_CALLBACK_OUTGOING_FROM"];

    $all_incoming_msns = implode(" ", $all_incoming_msns); # Alle MSNs aus dem Array zusammenfhren, da ein Key auch mehrere MSNs getrennt durch Leerzeichen enthalten kann.
    $all_incoming_msns = explode(" ", $all_incoming_msns); # Es soll aber jede MSN in einem Keywert stehen,
    $all_incoming_msns = array_unique($all_incoming_msns); # weil man dann die Doubletten besser rausfiltern kann.
    $all_incoming_msns = removeEmptyValues($all_incoming_msns); # leere MSN entfernen

    $all_outgoing_msns = implode(" ", $all_outgoing_msns); # Alle MSNs aus dem Array zusammenfhren, da ein Key auch mehrere MSNs getrennt durch Leerzeichen enthalten kann.
    $all_outgoing_msns = explode(" ", $all_outgoing_msns); # Es soll aber jede MSN in einem Keywert stehen,
    $all_outgoing_msns = array_unique($all_outgoing_msns); # weil man dann die Doubletten besser rausfiltern kann.
    $all_outgoing_msns = removeEmptyValues($all_outgoing_msns); # leere MSN entfernen

    $all = array_merge($all_incoming_msns, $all_outgoing_msns);
    $all = array_unique($all); # weil man dann die Doubletten besser rausfiltern kann.
    $all = removeEmptyValues($all);

    return array ($all_outgoing_msns, $all_incoming_msns, $all);

}
?>
