<?php
/**
 *---------------------------------------------------------------------------
 * /var/lib/asterisk/config/writeIncoming.php - Write incoming context
 *
 * Creation:     2005-10-28 fw
 * Last Update:  $Id: writeIncoming.php 8207 2014-12-17 12:12:00Z holbru $
 *
 * Copyright (c) 2005 Fabian Wolter, fabian(at)eisfair(dot)org
 * Copyright (c) 2014-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------
 */

function writeIncoming($msns, $type) {
    global $VARS;

    if (empty($wcContent)) {
        $wcContent = '';
    }
    $typeUpper = strtoupper($type);

    foreach ($msns as $keyMsn => $msn) {

        unset ($netbiosNames);
        unset ($internal_numbers_dial);
        unset ($internal_numbers_vbox);
        unset ($exec_cmds);
        unset ($keys);
        unset ($extraDialAttrib);
        unset ($redirectNumber);
        unset ($vboxTimeout);
        unset ($vbox_cmd);
        unset ($dial_cmd);
        unset ($hangup);

        if ($VARS["ASTERISK_VBOX_AUTODETECT_CID"] == "yes" && $VARS["ASTERISK_DIALPREFIX_VBOX"] == $msn)
            continue;

        if ($type == "sip") {
            $varName = "SIP";
            $typeUpper = $varName;
            $sipName = $msn["NAME"];
            $sipUser = $msn["USER"];
            $exten = $msn["CALLERID"];
            $searchToken = $sipName;
            $searchIn = "RING_ON_SIP";
        }
        elseif ($type == "capi") {
            $varName = "MSN";
            $typeUpper = "CAPI";
            $exten = $msn;
            $searchToken = $exten;
            $searchIn = "RING_ON_CAPI";

            /**
             * Falls ein entsprechender Eintrag in ASTERISK_MSN
             * vorhanden ist, wird $msn in ein Array umgewandelt
             */
            if (!empty($VARS['ASTERISK_MSN'])) {
                $key = multi_array_search($exten, $VARS['ASTERISK_MSN'], 'NUMBER', TRUE);

                if ($key !== false) {
                    $msn = $VARS['ASTERISK_MSN'][$key[0]];

                    /**
                    * overwrite $keyMsn of foreach()
                    */
                    $keyMsn = $key[0];
                }
            }
        }
        elseif ($type == "iax") {
            $varName = "IAX";
            $typeUpper = $varName;
            $iaxName = $msn["NAME"];
            $iaxUser = $msn["USER"];
            $exten = $msn["CALLERID"];
            $searchToken = $iaxName;
            $searchIn = "RING_ON_IAX";
        }

        /**
         * Pruefen ob der Account/die MSN fr eingehende callback
         * Anrufe verwendet werden soll
         */
        if ($VARS["ASTERISK_CALLBACK_LISTEN_NUMBER"] == $exten)
            continue;

        $keys = multi_array_search($searchToken, $VARS["ASTERISK_PHONES"], $searchIn);

        if ($keys === FALSE) {
            continue; # Falls eine MSN definiert wurde, die nicht in ASTERISK_PHONES_n_[$searchIn] vorkommt soll uebersprungen werden
        }
        foreach ($keys as $key) {
            #if(!ereg("RING_ON", $value[0]) && !ereg("RING_ON", $value[1])) continue; # $value kann auch gleich OUTGOING_MSN sein, das darf hier nicht verwendet werden

            $internal_numbers_dial[] = getDialCmd($VARS["ASTERISK_PHONES"][$key]["TYPE"], $VARS["ASTERISK_PHONES"][$key]["MSN"], $key);

            #if ($VARS["ASTERISK_PHONES"][$key]["TYPE"] == "disa" && $VARS["ASTERISK_PHONES"][$key]["RING_ON_CAPI"] != "") {
            #    err("ASTERISK_PHONES_".$key."_RING_ON_CAPI must be empty if using DISA in ASTERISK_PHONES_".$key."_TYPE", __LINE__);
            #}

            if ($VARS["ASTERISK_PHONES"][$key]["TYPE"] == "disa" && $VARS["ASTERISK_PHONES"][$key]["MSN"] == $exten && $type == "capi")
                return;

            if (!empty ($VARS["ASTERISK_PHONES"][$key]["EXEC_ON_RING"])) # Sammle alle Befehle aus EXEC_ON_RING wenn diese nicht leer ist und die aktuelle $msn in RING_ON_CAPI steht
                $exec_cmds[] = $VARS["ASTERISK_PHONES"][$key]["EXEC_ON_RING"];

            if (!empty ($VARS["ASTERISK_PHONES"][$key]["NETBIOS_NAME"])) # Sammle alle Befehle aus EXEC_ON_RING wenn diese nicht leer ist und die aktuelle $msn in RING_ON_CAPI steht
                $netbiosNames[] = $VARS["ASTERISK_PHONES"][$key]["NETBIOS_NAME"];

        }

        if (isset ($netbiosNames))
            $netbiosNames = array_unique($netbiosNames);

        if (isset ($internal_numbers_dial))
            $dial_cmd = implode("&", $internal_numbers_dial); # Dial command zusammenstellen

        else
            continue;

        ### Validate Vboxes
        if (isset($msn['VBOX']) && $msn['VBOX'] == 'yes') {
            $vboxValid = TRUE;

            if ($msn['VBOX_NUMBER'] == '') {
                err("ASTERISK_".$varName."_".$keyMsn."_VBOX_NUMBER is empty! Skipping...", __LINE__);
                $vboxValid = FALSE;
            }

            if ($msn["VBOX_ANSWER_AFTER"] == "") {
                err("ASTERISK_".$varName."_".$keyMsn."_VBOX_ANSWER_AFTER is empty! Skipping...", __LINE__);
                $vboxValid = FALSE;
            }

            if ($msn["REDIRECT_SCHEME"] != "") {
                err("ASTERISK_".$varName."_".$keyMsn."_REDIRECT_SCHEME is not empty and ASTERISK_".$varName."_".$keyMsn."_VBOX is set to 'yes'. Please specify the vbox in the appropriate redirect scheme (ASTERISK_REDIRECT_".$msn["REDIRECT_SCHEME"]."_SCHEME_n_TARGET_m_TYPE='vbox') or disable the redirect scheme (ASTERISK_".$varName."_".$keyMsn."_REDIRECT_SCHEME='').", __LINE__);
                $vboxValid = FALSE;
            }
        } else {
            $vboxValid = FALSE;
        }

        $e = new exten;

        $e->setExten($exten);

        if ($type == "capi") {
            "; Incoming calls over CAPI MSN $exten\n";
        }
        elseif ($type == "sip") {
            "; Incoming calls over SIP-Account: $sipName ($sipUser)\n";
        }
        elseif ($type == "iax") {
            "; Incoming calls over IAX-Account: $iaxName ($iaxUser)\n";
        }

        $e->setCmd("Ringing");

        $e = writeResolveNumbers($e);

        if (is_array($msn)) { # $msn ist _kein_ Array, wenn der Typ CAPI ist und keine entsprechenden Eintrge in ASTERISK_MSN_* vorhanden sind

            if ($msn["EXEC_ON_RING"] != "") {
                $e->setCmd("System", $msn["EXEC_ON_RING"]);
            }
        }

        if ($VARS["ASTERISK_RECORD_CONVERSATION"] == "yes") {
            $e->setCmd("Monitor", "wav,\${UNIQUEID},bm");
        }

        if (!empty($netbiosNames) && count($netbiosNames) > 0) {
            foreach ($netbiosNames as $netbiosName) {
                $e->setCmd("System", "echo \"Eingehender Anruf von \${CALLERID(number)} \${CALLERID(name)}\" , smbclient -U \"Asterisk Telefonserver\" -M \"$netbiosName\" &");
            }
        }

        if (!is_file("/var/lib/asterisk/vbox")) {

            if ($VARS["ASTERISK_DIALPREFIX_".$typeUpper] != "" && $VARS["ASTERISK_DIALPREFIX_SHOW_".$typeUpper] == "yes") {
                $tmpprio = $e->prio + 2;

                $e->setCmd("GotoIf", "\$[\"\${CALLERIDNUM}\" = \"\"]?$tmpprio");
                $e->setCmd("Set", "CALLERID(number)=".$VARS["ASTERISK_DIALPREFIX_".$typeUpper]."\${CALLERIDNUM}");
            }

            if ($vboxValid) {
                $localdialtimeout = $msn["VBOX_ANSWER_AFTER"];
            }
            elseif (!empty ($msn["REDIRECT_SCHEME"])) {
                $localdialtimeout = $VARS["ASTERISK_REDIRECT"][$msn["REDIRECT_SCHEME"]]["SCHEME"][1]["EXECUTE_AFTER"];
            }

            if (empty ($localdialtimeout)) {
                $localDialTimeoutString = "\${localdialtimeout}";
            } else {
                $localDialTimeoutString = $localdialtimeout;
            }

            /**
             * $msn ist _kein_ Array, wenn der Typ CAPI ist und keine
             * entsprechenden Eintrge in ASTERISK_MSN_* vorhanden sind
             */
            if (is_array($msn) && $msn['REDIRECT_SCHEME'] != '') {
                $result = writeRedirection($msn['REDIRECT_SCHEME'], $e, 'incoming', $type, $exten, 'before');

                if (is_object($result)) {
                    $e = $result;
                } else {
                    err('ASTERISK_'.$typeUpper.'_'.$keyMsn.'_REDIRECT_SCHEME: '.$result, __LINE__);
                }
            }

            if (empty($extraDialAttrib)) {
                $extraDialAttrib = '';
            }

            $e->setCmd("Dial", "$dial_cmd,$localDialTimeoutString,".OPTIONS_INCOMING."$extraDialAttrib");


            /**
             * $msn ist _kein_ Array, wenn der Typ CAPI ist und keine
             * entsprechenden Eintrge in ASTERISK_MSN_* vorhanden sind
             */
            if (is_array($msn)) {
                if ($msn["REDIRECT_SCHEME"] != "" && is_object($result)) {
                    $e = writeRedirection($msn["REDIRECT_SCHEME"], $e, "incoming", $type, $exten);
                    $e->setHangup(TRUE);
                } else {
                    if ($vboxValid) {
                        $vbox_cmd = $msn["VBOX_NUMBER"];

                        //                    $e->setCmd("Wait", "0.5", $e->prio);
                        //                    $e->setCmd("VoiceMail", VBOX_OPTIONS."u".$vbox_cmd, $e->prio + 1);
                        //                    $tmpprio = $e->prio + 100;
                        //                    $e->setCmd("Wait", "0.5", $tmpprio);
                        //                    $e->setCmd("VoiceMail", VBOX_OPTIONS."b".$vbox_cmd, $tmpprio +1);
                        //                    $tmpprio = $e->prio + 200;
                        //                    $e->setCmd("Wait", "0.5", $tmpprio);
                        //                    $e->setCmd("VoiceMail", VBOX_OPTIONS."u".$vbox_cmd, $tmpprio +1);
                        //                    $e->prio++;
                        //                    $e->prio++;
                        $e->setCmd("Goto", $exten."-\${DIALSTATUS},1");
                        $e->prio = 1;
                        $e->setExten($exten."-CHANUNAVAIL");
                        $e->setCmd("Wait", "0.5");
                        $e->setCmd("VoiceMail", VBOX_OPTIONS."b".$vbox_cmd);
                        $e->setHangup(TRUE);
                        $e->prio = 1;
                        $e->setExten($exten."-BUSY");
                        $e->setCmd("Wait", "0.5");
                        $e->setCmd("VoiceMail", VBOX_OPTIONS."b".$vbox_cmd);
                        $e->setHangup(TRUE);
                        $e->prio = 1;
                        $e->setExten($exten."-CONGESTION");
                        $e->setCmd("Wait", "0.5");
                        $e->setCmd("VoiceMail", VBOX_OPTIONS."b".$vbox_cmd);
                        $e->setHangup(TRUE);
                        $e->prio = 1;
                        $e->setExten("_".$exten."-.");
                        $e->setCmd("Wait", "0.5");
                        $e->setCmd("VoiceMail", VBOX_OPTIONS."u".$vbox_cmd);
                        $e->setHangup(TRUE);

                        $hangup = TRUE;
                    } else
                        $e->setHangup(TRUE);
                }
            } else
                $e->setHangup(TRUE);

            $wcContent .= $e->getData();

        } else
            $wcContent .= $e->getData()."\n";

    }
    return $wcContent;
}
?>
