<?php
/**
 *---------------------------------------------------------------------------
 * /var/lib/asterisk/config/writeContexts.php - Contains functions for writing the incoming, outgoing and redirection parts
 *
 * Creation:     2005-10-28 fw
 * Last Update:  $Id: writeMisc.php 8194 2014-12-07 22:11:17Z holbru $
 *
 * Copyright (c) 2005 Fabian Wolter, fabian(at)eisfair(dot)org
 * Copyright (c) 2014-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------
 */

function writeOutgoingCt($ct, $dialprefix, $type) {

    if (empty($wcContent)) {
        $wcContent = '';
    }

    $typeUpper = strtoupper($type);

    if ($ct['ALLOW_' . $typeUpper] == 'yes' &&
        $ct['OUTGOING_' . $typeUpper] != '') {

        $e = new exten;
        $e->setExten('_' . $dialprefix . '.');

        $wcContent .= '; Outgoing calls over '.$typeUpper."\n";

        $acc = get_acc_by_varname($type, $ct['OUTGOING_' . $typeUpper], 'NAME');

        if (!empty($acc['USER'])) {
            $accuser = $acc['USER'];
        }

        /**
         *  Sonderbehandlung fuer T-Online
         */
        if (is_integer(strpos($ct['OUTGOING_SIP'], 't-online')) &&
            is_integer(strpos($accuser, ':'))) {
            list ($tonlNummer) = explode(":", $accuser);
            $e->setCmd('Set', 'CALLERID(number)' . $tonlNummer);
        }

        if ($type == 'sip')
            $dialCmd = "SIP/\${EXTEN" . CUT_SIP . '}@' . $ct['OUTGOING_SIP'] . ',' . DIALTIMEOUT;
        else
            $dialCmd = 'IAX2/' . $ct['OUTGOING_IAX'] . "/\${EXTEN" . CUT_IAX . '},' . DIALTIMEOUT;

        $e->setCmd('Dial', $dialCmd);
        $e->setHangup();
        return $wcContent . $e->getData();

    }
}

function writeIvr($ivr, $n) {

    if (empty($wcContent)) {
        $wcContent = '';
    }

    $wcContent .= "[IVR$n]\n";
    $e = new exten;

    $e->setExten("s");
    $e->setCmd("Answer");
    $e->setCmd("SetMusicOnHold", "default");
    $e->setCmd("Set", "TIMEOUT(digit)=5");
    $e->setCmd("Set", "TIMEOUT(response)=10");
    $e->setCmd("Background", $ivr["INSTRUCTIONS"]);
    $wcContent .= $e->getData();

    $wcContent .= "\n";

    foreach ($ivr["ENTRY"] as $key => $entry) {
        $wcContent .= "; User pressed " . $entry["NUMBER"] . "\n";

        $e = new exten;
        $e->setExten($entry["NUMBER"]);

        if ($entry["TYPE"] == "exec") {
            $e->setCmd("System", $entry["VALUE"]);
            $e->setHangup();
        }
        elseif ($entry["TYPE"] == "dial") {
            $e->setCmd("Goto", "outgoing," . $entry["VALUE"] . ",1");
        }
        elseif ($entry["TYPE"] == "vbox") {
            $e->setCmd("Wait", "0.5");
            $e->setCmd("VoiceMail", VBOX_OPTIONS . "u" . $entry["VALUE"]);
        } else {
            err("ASTERISK_IVR_" . $n . "_ENTRY_" . $key . "_TYPE invalid value: " . $entry["VALUE"], __LINE__);
        }

        $wcContent .= $e->getData();
        $wcContent .= "\n";
    }
    return $wcContent;
}

function writeVbox($msn, $auto = FALSE) {
    global $VARS;

    if (empty($wcContent)) {
        $wcContent = '';
    }

    $e = new exten;

    if ($auto) {
        $e->setExten($VARS["ASTERISK_DIALPREFIX_VBOX"]);
        $wcContent .= "; VBOX menus\n";
        $e->setCmd("VoicemailMain", "s\${CALLERID(number)}");
    } else {
        $e->setExten($VARS["ASTERISK_DIALPREFIX_VBOX"] . $msn);
        $wcContent .= "; VBOX menu for voicebox number $msn\n";
        $e->setCID($msn);
        $e->setCmd("VoicemailMain", "s$msn");
        $e->setCID("");
        $e->prio--;
        $e->setCmd("VoicemailMain", $msn);
    }

    $e->setHangup();
    $wcContent .= $e->getData();

    return $wcContent;
}

function writeRedirection($schemeNumber, $e, $direction, $type, $msn, $time = "") {
    global $VARS, $globalRedirectContextHelper;

    if (!isset ($VARS["ASTERISK_REDIRECT"][$schemeNumber]["SCHEME"])) {
        return "The redirect scheme number $schemeNumber doesn't exist (ASTERISK_REDIRECT_" . $schemeNumber . "_SCHEME_*)'";
    }
    $schemes = $VARS["ASTERISK_REDIRECT"][$schemeNumber]["SCHEME"];

    if ($time == "before" && count($schemes) > 0) {
        $dialPrio = 1;
        foreach ($schemes as $scheme) {
            $dialPrio += count($scheme["EXECUTE_BETWEEN"]);
        }
        $dialPrio += $e->prio - 1;
    }

    for ($i = 1; $i <= count($schemes); $i++) {
        unset ($dialParts);
        unset ($vboxParts);
        $scheme = $schemes[$i];

        #if ($time == "before") {
        if ($schemes[$i +1]["EXECUTE_AFTER"] != "") # Fr den Dialtimeout muss das EXECUTE_AFTER aus dem nchsten Element des Arrays bestimmt werden. Ist das aktuelle Element das letzte Element des Array, wird der Standardwer benutzt.
            $localDialTimeout = $schemes[$i +1]["EXECUTE_AFTER"];
        else
            $localDialTimeout = DIALTIMEOUT;

        $bVbox = FALSE;
        $bOther = FALSE;
        if (isset ($scheme["TARGET"])) {
            foreach ($scheme["TARGET"] as $targetKey => $target) {

                unset ($dialCmd);
                unset ($vboxCmd);

                if ($target["TYPE"] == "internal") {
                    $result1 = multi_array_search($target["ADDR"], $VARS["ASTERISK_PHONES"], "MSN", TRUE);
                    $result2 = multi_array_search($target["ADDR"], $VARS["ASTERISK_IVR"], "NUMBER", TRUE);

                    if ($result1 === FALSE && $result2 === FALSE)
                        return $target["ADDR"] . " is not a valid internal MSN (set in ASTERISK_REDIRECT_" . $schemeNumber . "_SCHEME_" . $i . "_TARGET_" . $targetKey . "_ADDR)";

                    if ($result1 !== FALSE)
                        $key = $result1[0];

                    if ($result2 !== FALSE)
                        $key = $result2[0];
                }
                //                    $type2 = $VARS["ASTERISK_PHONES"][$key]["TYPE"];
                //                } else
                $type2 = $target["TYPE"];

                if ($target["TYPE"] == "vbox")
                    $bVbox = TRUE;
                else
                    $bOther = TRUE;

                if (($target["TYPE"] == "vbox" && $bOther) || ($target["TYPE"] != "vbox" && $bVbox)) {
                    err("You cannot define a vbox and another type of direction in the same redirection scheme target.\n(ASTERISK_REDIRECT_$schemeNumber_SCHEME_$i_TARGET_*)");
                    continue 2;
                }

                if ($target["TYPE"] == "vbox") {
                    $vboxParts[] = $target["ADDR"];
                } else {
                    $dialParts[] = getDialCmd($type2, $target["ADDR"], $key, $localDialTimeout, $target["FROM"]);
                }
            }
        }

        if (isset ($dialParts))
            $dialCmd = implode("&", $dialParts);
        if (isset ($vboxParts))
            $vboxCmd = implode("&", $vboxParts);
        #}

        if ($scheme["EXECUTE_BETWEEN_N"] <= 0 && $time != "before") {
            #$e->prio += 100;

            if (isset ($dialParts)) {
                $e->setCmd("Dial", $dialCmd);
            }
            elseif (isset ($vboxParts)) {
                $e->setCmd("Wait", "0.5");
                $e->setCmd("VoiceMail", VBOX_OPTIONS . "u" . $vboxCmd);
            } else
                err("Error while writing dial/vbox parts of the redirection ASTERISK_REDIRECT_" .
                $schemeNumber . "_SCHEME_" . $i . "_TARGET_*", __LINE__);
        } else {
            for ($j = 1; $j <= count($scheme["EXECUTE_BETWEEN"]); $j++) {
                $desc = "$direction-$type-$msn-$i";

                if ($time == "before") {
                    $desc .= "-setDialTimeout";
                }

                $e->setCmd("GotoIfTime", $scheme["EXECUTE_BETWEEN"][$j]["TIME"] . "?RedirectionContextHelper_$desc," . $e->exten . ",1");

                if ($j < count($scheme["EXECUTE_BETWEEN"]))
                    continue;

                if ($time == "before") {
                    $globalRedirectContextHelper[$desc][$j]["setDialTimeout"] = TRUE;
                    $globalRedirectContextHelper[$desc][$j]["prio"] = $dialPrio +1;
                } else {
                    $globalRedirectContextHelper[$desc][$j]["prio"] = $e->prio;
                }

                if (isset ($dialCmd)) {
                    $globalRedirectContextHelper[$desc][$j]["dialCmd"] = $dialCmd;
                }
                elseif (isset ($vboxCmd)) {
                    $globalRedirectContextHelper[$desc][$j]["vboxCmd"] = $vboxCmd;
                }

                $globalRedirectContextHelper[$desc][$j]["exten"] = $e->exten;

                $globalRedirectContextHelper[$desc][$j]["msn"] = $msn;
                $globalRedirectContextHelper[$desc][$j]["localDialTimeout"] = $schemes[$i]["EXECUTE_AFTER"];
            }
            if ($time == "before" && $schemes[$i +1]["EXECUTE_AFTER"] != "") {
                $e->setCmd("Set", "localdialtimeout=" . $schemes[$i +1]["EXECUTE_AFTER"]);

            }
        }

    }
    //    for ($j = 1; $j <= count($schemes) && $time == "before"; $j ++) {
    //        if ($schemes[$j]["EXECUTE_BETWEEN_N"] === 0) {
    //            $e->setCmd("SetVar", "localdialtimeout=".$schemes[$j]["EXECUTE_AFTER"]);
    //            echo "localdialtimeout=".$schemes[$j]["EXECUTE_AFTER"];
    //            break;
    //        }
    //    }
    return $e;
}

function writeRedirectionContextHelper() {
    global $globalRedirectContextHelper;

    if (empty($wcContent)) {
        $wcContent = '';
    }

    if (isset ($globalRedirectContextHelper)) {
        foreach ($globalRedirectContextHelper as $desc => $value1) {
            foreach ($value1 as $key => $value) {
                $dialCmd = $value["dialCmd"];
                $vboxCmd = $value["vboxCmd"];
                $exten = $value["exten"];
                $nextPrio = $value["prio"];
                $localDialTimeout = $value["localDialTimeout"];
                list ($direction) = explode("-", $desc);
                $e = new exten;
                $e->setExten($exten);

                $wcContent .= "[RedirectionContextHelper_$desc]\n";

                if ($value["setDialTimeout"]) {
                    $e->setCmd("Set", "localdialtimeoutSet=1");
                    $e->setCmd("Set", "localdialtimeout=$localDialTimeout");
                }
                elseif (isset ($value["nextBetween"])) {
                    $tmp = $key +1;
                    $e->setCmd("GotoIfTime", $value["nextBetween"] . "?$desc,$exten,1");

                } else {
                    if (empty ($dialCmd)) {
                        $e->setCmd("Wait", "0.5");
                        $e->setCmd("VoiceMail", VBOX_OPTIONS . "u" . $vboxCmd);
                    }
                    elseif (empty ($vboxCmd)) {
                        $e->setCmd("Dial", "$dialCmd,$localDialTimeout," . OPTIONS_INCOMING);
                    } else
                        err("Wether dialCmd nor vboxCmd is set.", __LINE__);
                }

                $e->setCmd("Goto", "$direction,$exten,$nextPrio");
                $wcContent .= $e->getData();
                $wcContent .= "\n";
            }
        }
    }
    return $wcContent;
}
?>
