<?php
/**
 *---------------------------------------------------------------------------
 * /var/lib/asterisk/config/writeOutgoing.php - Write outgoing context
 *
 * Creation:     2005-10-28 fw
 * Last Update:  $Id: writeOutgoing.php 9283 2016-04-14 16:59:35Z holbru $
 *
 * Copyright (c) 2005 Fabian Wolter, fabian(at)eisfair(dot)org
 * Copyright (c) 2014-2019 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------
 */

function writeOutgoing($phone, $type) {
    global $VARS;

    if (empty($wcContent)) {
        $wcContent = '';
    }

    $typeUpper = strtoupper($type);

    if ($type == "sip")
        $cut = CUT_SIP;
    elseif ($type == "iax") $cut = CUT_IAX;

    if ($phone["OUTGOING_".$typeUpper] != "") {

        $wcContent .= "; Outgoing calls over $typeUpper\n";

        $acc = get_acc_by_varname($type, $phone["OUTGOING_".$typeUpper], "NAME", "array"); # Anhand des Namens des Sipaccounts die Nummer (Username) herausfinden

        $e = new exten;

        if ($phone["OUTGOING_".$typeUpper] != "" && !empty ($acc)) {
            $accnumber = $acc["CALLERID"];
            $accfallback = $acc["FALLBACK"];

            if (isset ($accfallback)) {

                $fallbacks = explode(" ", $accfallback);

                foreach ($fallbacks as $fallback) {

                    if (empty ($fallback))
                        continue;

                    list ($channelType, $outgoing) = explode(":", $fallback);

                    if (($channelType == "sip" || $channelType == "iax") && $outgoing != "") {

                        $acc2 = get_acc_by_varname($channelType, $outgoing, "NAME", "array", FALSE);

                        if ($acc2 === FALSE) {
                            err("The $channelType-Account \"".$outgoing."\" given in ASTERISK_".$typeUpper."_".$acc["key"]."_FALLBACK doesn't exist", __LINE__);

                            $VARS["ASTERISK_".$typeUpper][$acc["key"]]["FALLBACK"] = preg_replace("/$channelType:$outgoing/", "", $VARS["ASTERISK_".$typeUpper][$acc["key"]]["FALLBACK"]); # To avoid further error messages

                            continue;
                        }
                    }

                    if ($channelType == "sip") {
                        $fallbackDialCmds[] = "SIP/\${areaCode}\${EXTEN".$cut."}@$outgoing";
                    }
                    elseif ($channelType == "iax") {
                        $fallbackDialCmds[] = "IAX2/$outgoing/\${areaCode}\${EXTEN".$cut."}";
                    }
                    elseif ($channelType == "capi") {
                        $fallbackDialCmds["No".$outgoing] = "CAPI/contr1/$outgoing:\${EXTEN".$cut."}/b";
                        $all_outgoing_msns[] = $outgoing;
                    }
                    elseif (preg_match("/hfcs/", $channelType)) {
                        $fallbackDialCmds["No".$outgoing] = "Dahdi/g".getHfcsGroup($channelType)."/\${EXTEN".$cut."}";
                    } else
                        err("Fallbacktype ($fallback) unkown", __LINE__);
                }
            }

            if (!empty ($accnumber) && $acc !== FALSE) {

                if ($type == "sip")
                    $tmp = DIALPREFIX_SIP;
                elseif ($type == "iax") $tmp = DIALPREFIX_IAX;

                $e->setExten("_$tmp.");
                $e->setCID($phone["MSN"]);
                $e->setCmd("Ringing", "");

                if (is_integer(strpos($acc["NAME"], "t-online")) && is_integer(strpos($acc["USER"], ":"))) { # Sonderbehandlung fr T-Offline
                    list ($tonlNummer) = explode(":", $acc["USER"]);
                    list ($tonlName) = explode(":", $acc["NAME"]);
                    $e->setCmd("Set", "CALLERID(number)=".$tonlNummer);
                    $e->setCID($accnumber);
                    $e->setCmd("Set", "CALLERID(name)=".$tonlName);
                } else {
                    $e->setCmd("Set", "CALLERID(number)=".$accnumber);
                    $e->setCID($accnumber);
                    $e->setCmd("Set", "CALLERID(name)=".$phone["OUTGOING_NAME"]);
                }

                $priotmp = $e->prio + 2;

                if ($VARS["ASTERISK_AREA_CODE"] != "") {
                    if ($type == "sip")
                        $cut_tmp = DIALPREFIX_SIP;
                    else
                        $cut_tmp = DIALPREFIX_IAX;

                    if ($cut_tmp == "x")
                        $cut_new = ":0";
                    else {
                        if ($type == "sip")
                            $cut_new = CUT_SIP;
                        else
                            $cut_new = CUT_IAX;
                    }

                    $e->setCmd("GotoIf", "\$[\"\${EXTEN$cut_new:1}\" = \"0\"]?$priotmp");
                    $e->setCmd("Set", "areaCode=".$VARS["ASTERISK_AREA_CODE"]);
                }

                if ($VARS["ASTERISK_RECORD_CONVERSATION"] == "yes") {
                    $e->setCmd("Monitor", "wav,\${UNIQUEID},bm");
                }

                if ($type == "sip")
                    $dialCmd = "SIP/\${areaCode}\${EXTEN".CUT_SIP."}@".$phone["OUTGOING_SIP"].",".DIALTIMEOUT.",".OPTIONS_OUTGOING;
                else
                    $dialCmd = "IAX2/".$phone["OUTGOING_IAX"]."/\${areaCode}\${EXTEN".CUT_IAX."},".DIALTIMEOUT.",".OPTIONS_OUTGOING;

                $e->setCmd("Dial", $dialCmd);

                if (isset ($fallbackDialCmds)) {
                    foreach ($fallbackDialCmds as $key => $fallbackDialCmd) {
                        if (preg_match("/CAPI|Dahdi/", $fallbackDialCmd)) { # CID nur setzen, wenn die Gespraeche nicht ueber SIP rausgehen
                            $e->setCmd("Set", "CALLERID(number)=".substr($key, 2));
                        }

                        $e->setCmd("Dial", $fallbackDialCmd.",".DIALTIMEOUT.",".OPTIONS_OUTGOING);
                    }
                }
                $e->setHangup();
                $wcContent .= $e->getData();
                $wcContent .= "\n";
            }
        } else {
            $e = new exten;

            if ($type == "sip")
                $tmp = DIALPREFIX_SIP;
            else
                $tmp = DIALPREFIX_IAX;

            $e->setExten("_".$tmp.".");
            $e->setCID($phone["MSN"]);
            $e->setCmd("Playback", "ss-noservice");
            $e->setHangup(TRUE);
            $wcContent .= $e->getData();
        }
    }

    return $wcContent;
}
?>
